@@ -27,7 +27,10 @@
 class KUrl;
 
 namespace KDevelop {
+
+class IVariableController;
 class IBreakpointController;
+class IStackController;
 class Breakpoint;
 class StackModel;
 
@@ -55,8 +58,6 @@ public:
      */
     virtual DebuggerState state() const = 0;
 
-    virtual StackModel* stackModel() const = 0;
-
     /**
      * Should return if restart is currently available
      */
@@ -67,7 +68,9 @@ public:
      */
     bool isRunning() const;
     
-    IBreakpointController *breakpointController();
+    IBreakpointController *breakpointController() const;
+    IVariableController *variableController() const;
+    IStackController *stackController() const;
 
 public Q_SLOTS:
     virtual void restartDebugger() = 0;
@@ -93,6 +96,8 @@ Q_SIGNALS:
 protected:
 
     IBreakpointController *m_breakpointController;
+    IVariableController *m_variableController;
+    IStackController *m_stackController;
 };
 
 }
