@@ -21,8 +21,8 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#ifndef KDEVBACKGROUNDPARSER_H
-#define KDEVBACKGROUNDPARSER_H
+#ifndef BACKGROUNDPARSER_H
+#define BACKGROUNDPARSER_H
 
 #include <QtCore/QMap>
 #include <QtCore/QPair>
@@ -30,9 +30,9 @@
 #include <QtCore/QHash>
 #include <QtCore/QPointer>
 
-#include <KUrl>
+#include <KDE/KUrl>
 
-#include <KTextEditor/SmartRangeWatcher>
+#include <KDE/KTextEditor/SmartRangeWatcher>
 
 #include "../languageexport.h"
 #include <interfaces/istatus.h>
@@ -108,6 +108,12 @@ public:
      */
     Q_SCRIPTABLE void addManagedTopRange(const KUrl& document, KTextEditor::SmartRange* range);
 
+    /**
+     * Returns all documents that were added through addManagedTopRange. This is typically the currently
+     * open documents.
+     */
+    Q_SCRIPTABLE QList<KUrl> managedDocuments();
+    
     /**
      * Remove an associated top \a range from modification watching.
      */
@@ -152,7 +158,7 @@ public Q_SLOTS:
      * @p priority A value that manages the order of parsing. Documents with lowest priority are parsed first.
      * @param notifyReady An optional pointer to a QObject that should contain a slot
      *                    "void updateReady(KDevelop::IndexedString url, KDevelop::ReferencedTopDUContext topContext)".
-     *                    The notification is guaranteed to be called once for each call to updateContextForUrl. The given top-context
+     *                    The notification is guaranteed to be called once for each call to addDocument. The given top-context
      *                    may be invalid if the update failed.
      */
     void addDocument(const KUrl& url, TopDUContext::Features features = TopDUContext::VisibleDeclarationsAndContexts, int priority = 0, QObject* notifyWhenReady = 0);
@@ -165,9 +171,10 @@ public Q_SLOTS:
     void addDocumentList(const KUrl::List& urls, TopDUContext::Features features = TopDUContext::VisibleDeclarationsAndContexts, int priority = 0);
 
     /**
-     * Removes the @p url from the queue.
+     * Removes the @p url that is registered for the given notification from the url.
+     * @param notifyWhenReady Notifier the document was added with
      */
-    void removeDocument(const KUrl& url);
+    void removeDocument(const KUrl& url, QObject* notifyWhenReady = 0);
 
     /**
      * Forces the current queue to be parsed.
@@ -187,6 +194,9 @@ public Q_SLOTS:
     ///Enables all processing of new jobs, equivalent to setNeededPriority(WorstPriority)
     void enableProcessing();
     
+    ///Returns true if the given url is queued for parsing
+    bool isQueued(KUrl url) const;
+    
 protected:
     void loadSettings(bool projectIsLoaded);
     void saveSettings(bool projectIsLoaded);
@@ -208,4 +218,3 @@ private:
 
 }
 #endif
-
