@@ -22,6 +22,8 @@
 #include "../languageexport.h"
 #include <language/duchain/indexedstring.h>
 
+#include <ksharedptr.h>
+
 class QString;
 
 namespace KTextEditor {
@@ -33,20 +35,29 @@ namespace KDevelop {
     class IndexedString;
     
 ///Allows getting code-lines conveniently, either through an open editor, or from a disk-loaded file.
-class KDEVPLATFORMLANGUAGE_EXPORT CodeRepresentation {
+class KDEVPLATFORMLANGUAGE_EXPORT CodeRepresentation : public QSharedData {
   public:
     virtual ~CodeRepresentation() {
     }
     virtual QString line(int line) const = 0;
     virtual int lines() const = 0;
     virtual QString text() const = 0;
+    virtual QString rangeText(KTextEditor::Range range) const;
     ///Overwrites the text in the file with the new given one
     ///@return true on success
     virtual bool setText(QString) = 0;
+    ///@return true if this representation represents an actual file on disk
+    virtual bool fileExists() = 0;
     
     ///Can be used for example from tests to disallow on-disk changes. When such a change is done, an assertion is triggered.
     ///You should enable this within tests, unless you really want to work on the disk.
     static void setDiskChangesForbidden(bool changesForbidden);
+   
+    ///Returns the specified name as a url for aritificial source code
+    ///suitable for code being inserted into the parser
+    static KUrl artificialUrl(const QString & name);
+ 
+    typedef KSharedPtr<CodeRepresentation> Ptr;
 };
 
 class KDEVPLATFORMLANGUAGE_EXPORT DynamicCodeRepresentation : public CodeRepresentation {
@@ -54,19 +65,25 @@ class KDEVPLATFORMLANGUAGE_EXPORT DynamicCodeRepresentation : public CodeReprese
       ///Used to group edit-history together. Call this before a bunch of replace(), and endEdit in the end.
       virtual void startEdit() = 0;
       virtual bool replace(const KTextEditor::Range& range, QString oldText, QString newText, bool ignoreOldText = false) = 0;
-      virtual QString rangeText(KTextEditor::Range range) const = 0;
       ///Must be called exactly once per startEdit()
       virtual void endEdit() = 0;
+    
+      typedef KSharedPtr<DynamicCodeRepresentation> Ptr;
 };
 
 ///Creates a code-representation for the given url, that allows conveniently accessing its data. Returns zero on failure.
-KDEVPLATFORMLANGUAGE_EXPORT CodeRepresentation* createCodeRepresentation(IndexedString url);
+KDEVPLATFORMLANGUAGE_EXPORT CodeRepresentation::Ptr createCodeRepresentation(IndexedString url);
+
+///@return true if an artificial code representation already exists for the specified URL
+KDEVPLATFORMLANGUAGE_EXPORT bool artificialCodeRepresentationExists(IndexedString url);
 
 ///An artificial source-code representation that can be used for testing.
 ///It inserts itself automatically into the code-representation framework,
 ///logically representing the contents of a file. Thus, it can be used for testing
 ///refactoring.
-class KDEVPLATFORMLANGUAGE_EXPORT InsertArtificialCodeRepresentation {
+///
+///@warning This object is not copyable
+class KDEVPLATFORMLANGUAGE_EXPORT InsertArtificialCodeRepresentation : public QSharedData {
     public:
         InsertArtificialCodeRepresentation(IndexedString file, QString text);
         ~InsertArtificialCodeRepresentation();
@@ -76,6 +93,8 @@ class KDEVPLATFORMLANGUAGE_EXPORT InsertArtificialCodeRepresentation {
         IndexedString m_file;
 };
 
+typedef KSharedPtr<InsertArtificialCodeRepresentation> InsertArtificialCodeRepresentationPointer;
+
 }
 
 
