@@ -631,6 +631,7 @@ DUContext::~DUContext( )
   top->m_dynamicData->clearContextIndex(this);
 
   Q_ASSERT(d_func()->isDynamic() == (!top->deleting() || !top->isOnDisk() || top->m_dynamicData->isTemporaryContextIndex(m_dynamicData->m_indexInTopContext)));
+  delete m_dynamicData;
 }
 
 QVector< DUContext * > DUContext::childContexts( ) const
@@ -762,8 +763,8 @@ void DUContext::findLocalDeclarationsInternal( const Identifier& identifier, con
 
      if(m_dynamicData->m_hasLocalDeclarationsHash) {
        //Use a special hash that contains all declarations visible in this context
-      QHash<Identifier, DeclarationPointer>::const_iterator it = m_dynamicData->m_localDeclarationsHash.find(identifier);
-      QHash<Identifier, DeclarationPointer>::const_iterator end = m_dynamicData->m_localDeclarationsHash.end();
+      QHash<Identifier, DeclarationPointer>::const_iterator it = m_dynamicData->m_localDeclarationsHash.constFind(identifier);
+      QHash<Identifier, DeclarationPointer>::const_iterator end = m_dynamicData->m_localDeclarationsHash.constEnd();
 
       for( ; it != end && it.key() == identifier; ++it ) {
         Declaration* declaration = (*it).data();
@@ -1036,8 +1037,8 @@ QList<Declaration*> DUContext::allLocalDeclarations(const Identifier& identifier
   QList<Declaration*> ret;
 
   if(m_dynamicData->m_hasLocalDeclarationsHash) {
-    QHash<Identifier, DeclarationPointer>::const_iterator it = m_dynamicData->m_localDeclarationsHash.find(identifier);
-    QHash<Identifier, DeclarationPointer>::const_iterator end = m_dynamicData->m_localDeclarationsHash.end();
+    QHash<Identifier, DeclarationPointer>::const_iterator it = m_dynamicData->m_localDeclarationsHash.constFind(identifier);
+    QHash<Identifier, DeclarationPointer>::const_iterator end = m_dynamicData->m_localDeclarationsHash.constEnd();
 
     for( ; it != end && it.key() == identifier; ++it )
       ret << (*it).data();
@@ -1883,7 +1884,7 @@ void DUContext::SearchItem::addToEachNode(SearchItem::PtrList other) {
 
 DUContext::Import::Import(DUContext* _context, const DUContext* importer, const SimpleCursor& _position) : position(_position) {
   if(_context && _context->owner() && (_context->owner()->specialization().index() || (importer && importer->topContext() != _context->topContext()))) {
-    m_declaration = _context->owner()->id(true);
+    m_declaration = _context->owner()->id();
   }else{
     m_context = _context;
   }
