@@ -268,7 +268,11 @@ struct QualifiedIdentifierItemRequest {
   const DynamicQualifiedIdentifierPrivate& m_identifier;
 };
 
-RepositoryManager< ItemRepository<ConstantQualifiedIdentifierPrivate, QualifiedIdentifierItemRequest>, false> qualifiedidentifierRepository("Qualified Identifier Repository");
+AbstractRepositoryManager* returnIdentifierRepository() {
+  return &identifierRepository;
+}
+
+RepositoryManager< ItemRepository<ConstantQualifiedIdentifierPrivate, QualifiedIdentifierItemRequest>, false> qualifiedidentifierRepository("Qualified Identifier Repository", 1, &returnIdentifierRepository);
 
 uint emptyConstantQualifiedIdentifierPrivateIndex() {
    static uint index = qualifiedidentifierRepository->index(DynamicQualifiedIdentifierPrivate());
