@@ -181,7 +181,7 @@ QString AbstractDeclarationNavigationContext::html(bool shorten)
   }
   
   if( shorten && !m_declaration->comment().isEmpty() ) {
-    QString comment = m_declaration->comment();
+    QString comment = QString::fromUtf8(m_declaration->comment());
     if( comment.length() > 60 ) {
       comment.truncate(60);
       comment += "...";
@@ -256,7 +256,7 @@ QString AbstractDeclarationNavigationContext::html(bool shorten)
   
   if( !shorten && (!m_declaration->comment().isEmpty() || doc) ) {
     modifyHtml() += "<br />";
-    QString comment = m_declaration->comment();
+    QString comment = QString::fromUtf8(m_declaration->comment());
     if(comment.isEmpty() && doc) {
       comment = doc->description();
       if(!comment.isEmpty()) {
@@ -567,8 +567,9 @@ void AbstractDeclarationNavigationContext::eventuallyMakeTypeLinks( AbstractType
     }
 
   } else {
-    if(idType)
+    if(idType) {
       kDebug() << "identified type could not be resolved:" << idType->qualifiedIdentifier() << idType->declarationId().isValid() << idType->declarationId().isDirect();
+    }
     modifyHtml() += typeHighlight(Qt::escape(type->toString()));
   }
 }
