@@ -272,6 +272,8 @@ public:
   ParsingEnvironmentFilePointer m_file;
   QList<ProblemPointer> m_problems;
 
+  KSharedPtr<IAstContainer> m_ast;
+  
   uint m_ownIndex;
 
   bool m_inDuChain;
@@ -756,8 +758,8 @@ void TopDUContextLocalPrivate::rebuildStructure(const TopDUContext* imported) {
       if(top == imported) {
         addImportedContextRecursion(top, imported, 1);
       }else{
-        RecursiveImports::const_iterator it2 = top->m_local->m_recursiveImports.find(imported);
-        if(it2 != top->m_local->m_recursiveImports.end()) {
+        RecursiveImports::const_iterator it2 = top->m_local->m_recursiveImports.constFind(imported);
+        if(it2 != top->m_local->m_recursiveImports.constEnd()) {
 
           addImportedContextRecursion(top, imported, (*it2).first + 1);
         }
@@ -772,8 +774,8 @@ void TopDUContextLocalPrivate::rebuildStructure(const TopDUContext* imported) {
       if(top == imported) {
         addImportedContextRecursion(top, imported, 1);
       }else{
-        RecursiveImports::const_iterator it2 = top->m_local->m_recursiveImports.find(imported);
-        if(it2 != top->m_local->m_recursiveImports.end()) {
+        RecursiveImports::const_iterator it2 = top->m_local->m_recursiveImports.constFind(imported);
+        if(it2 != top->m_local->m_recursiveImports.constEnd()) {
 
           addImportedContextRecursion(top, imported, (*it2).first + 1);
         }
@@ -863,6 +865,7 @@ void TopDUContext::setFeatures(Features features)
 {
   features = (TopDUContext::Features)(features & (~((uint)8))); //Remove the "Recursive" flag since that's only for searching
   features = (TopDUContext::Features)(features & (~ForceUpdateRecursive)); //Remove the update flags
+  features = (TopDUContext::Features)(features & (~AST)); //Remove the AST flag, it's only used while updating
   d_func_dynamic()->m_features = features;
 
   //Replicate features to ParsingEnvironmentFile
@@ -1450,14 +1453,6 @@ bool TopDUContext::isOnDisk() const {
   return m_dynamicData->isOnDisk();
 }
 
-IndexedString TopDUContext::language() const {
-  return d_func()->m_language;
-}
-
-void TopDUContext::setLanguage(IndexedString language) {
-  d_func_dynamic()->m_language = language;
-}
-
 void TopDUContext::clearUsedDeclarationIndices() {
   ENSURE_CAN_WRITE
   for(unsigned int a = 0; a < d_func()->m_usedDeclarationIdsSize(); ++a)
@@ -1488,7 +1483,7 @@ int TopDUContext::indexForUsedDeclaration(Declaration* declaration, bool create)
 if(!declaration)
   return std::numeric_limits<int>::max();
 
-  if(declaration->topContext() == this && !declaration->inSymbolTable()) {
+  if(declaration->topContext() == this && !declaration->inSymbolTable() && !m_dynamicData->isTemporaryDeclarationIndex(declaration->ownIndex())) {
     uint index = declaration->ownIndex();
     Q_ASSERT(!(index & (1<<31)));
     return (int)(index | (1<<31)); //We don't put context-local declarations into the list, that's a waste. We just use the mark them with the highest bit.
@@ -1501,6 +1496,7 @@ if(!declaration)
   uint size = d_func()->m_usedDeclarationIdsSize();
   const DeclarationId* ids = d_func()->m_usedDeclarationIds();
 
+  ///@todo Make m_usedDeclarationIds sorted, and find the decl. using binary search
   for(unsigned int a = 0; a < size; ++a)
     if(ids[a] == id) {
       index = a;
@@ -1551,6 +1547,22 @@ TopDUContext::Cache::~Cache() {
   delete d;
 }
 
+void TopDUContext::setAst(KSharedPtr<IAstContainer> ast)
+{
+  ENSURE_CAN_WRITE
+  m_local->m_ast = ast;
+}
+
+KSharedPtr<IAstContainer> TopDUContext::ast() const
+{
+  return m_local->m_ast;
+}
+
+void TopDUContext::clearAst(void)
+{
+  setAst(KSharedPtr<IAstContainer>(0));
+}
+
 IndexedString TopDUContext::url() const {
   return d_func()->m_url;
 }
