@@ -1,6 +1,7 @@
 /* This file is part of KDevelop
     Copyright 2006 Roberto Raggi <roberto@kdevelop.org>
     Copyright 2006-2008 Hamish Rodda <rodda@kde.org>
+    Copyright 2007-2008 David Nolden <david.nolden.kdevelop@art-master.de>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -58,16 +59,21 @@ class TypeExchanger;
  *  until we save it back to the static memory-region again.
  *
  * When creating an own type, you must:
- * - Implement equals(..), clone(), hash()
+ * - Implement equals(..), hash(). They should _fully_ distinguish all types, in regard to all stored information.
+ *  * implement a copy-constructor in which you copy the data from the source using copyData<YourType>()
+ *  * Implement clone() in which you use the copy-constructor to clone the type
  * - Add an enumerator "Identity" that contains an arbitrary unique identity value of the type
  * - Add a typedef "BaseType" that specifies the base type, which must be a type that also follows these rules(@todo)
  * - Register the type in a source-file using REGISTER_TYPE(..), @see typeregister.h
  * - Add a typedef "Data", that contains the actual data of the type using the mechanisms described in appendedlist.h.
  *   That data type must follow the same inheritance chain as the type itself, so it must be based on BaseType::Data.
  *   @see AbstractTypeData
- * - When creating a data object in a constructor, you must call setTypeClassId<YourType>() on that data to mark it.
+ * - When creating a data object in a constructor, use createData<YourType>()
+ * - Use TYPE_DECLARE_DATA(YourType) to declare the data access functions, and  then use d_func() and d_func_dynamic() to 
+ *   access your type data
+ * - Create a constructor that only takes a reference to the type data, and passes it to the parent type
  *
- *   Every type can have only one other type as base-class,
+ *   Every type can have only one other type as base-type,
  *   but it can have additional base-classes that are not a direct part of the type-system(@see IdentifiedType).
  *
  *  \sa appendedlist.h
@@ -91,24 +97,16 @@ public:
     ConstModifier               = 1 << 0,
     VolatileModifier            = 1 << 1,
     TransientModifier           = 1 << 2,
-    FinalModifier               = 1 << 3,///@todo  remove, is a property of a declaration, not of its type
-    AbstractModifier            = 1 << 4,///@todo  remove, is a property of a declaration, not of its type
-    NativeModifier              = 1 << 5,///@todo  remove, is a property of a declaration, not of its type
-    SynchronizedModifier        = 1 << 6,///@todo  remove, is a property of a declaration, not of its type
-    StrictFPModifier            = 1 << 7,
-    NewModifier                 = 1 << 8,
-    SealedModifier              = 1 << 9,
-    StaticModifier              = 1 << 10,///@todo  remove, is a property of a declaration, not of its type
-    ReadonlyModifier            = 1 << 11,///@todo  remove, is a property of a declaration, not of its type
-    OverrideModifier            = 1 << 12,///@todo  remove, is a property of a declaration, not of its type
-    UnsafeModifier              = 1 << 13,
-    FixedModifier               = 1 << 14,
-    ShortModifier               = 1 << 15,
-    LongModifier                = 1 << 16,
-    LongLongModifier            = 1 << 17,
-    SignedModifier              = 1 << 18,
-    UnsignedModifier            = 1 << 19,
-    LanguageSpecificModifier    = 1 << 20 //TODO make this support 64 bit values
+    NewModifier                 = 1 << 3,
+    SealedModifier              = 1 << 4,
+    UnsafeModifier              = 1 << 5,
+    FixedModifier               = 1 << 6,
+    ShortModifier               = 1 << 7,
+    LongModifier                = 1 << 8,
+    LongLongModifier            = 1 << 9,
+    SignedModifier              = 1 << 10,
+    UnsignedModifier            = 1 << 11,
+    LanguageSpecificModifier    = 1 << 12 //TODO make this support 64 bit values
   };
 
   /// Constructor.
@@ -134,6 +132,7 @@ public:
 
   /**
    * Visitor method.  Called by TypeVisitor to visit the type heirachy.
+   * Do not reimplement this, reimplement accept0 instead.
    *
    * \param v visitor which is calling this function.
    */
@@ -188,7 +187,9 @@ public:
     TypeArray     /**< an array */,
     TypeDelayed   /**< a delayed type */,
     TypeEnumeration /**< an enumeration type */,
-    TypeEnumerator /**< an enumerator type */
+    TypeEnumerator /**< an enumerator type */,
+    TypeAlias      /**< a type-alias type */,
+    TypeUnsure /**< may represent multiple different types */
   };
 
   /**
@@ -205,7 +206,8 @@ public:
   /**
    * Should, like accept0, be implemented by all types that hold references to other types.
    *
-   * \todo document function
+   * If this is called on one type, that type should call exchangeTypes(..) with all its referenced sub-types.
+   * The type itself does not recurse into the sub-types, that can be done by the exchanger itself if desired.
    * */
   virtual void exchangeTypes( TypeExchanger* exchanger );
 
@@ -217,8 +219,25 @@ public:
    * \param rhs data to copy
    * \returns copy of the data
    */
+
+  template<class Type>
+  static typename Type::Data& copyData(const typename Type::Data& rhs) {
+    size_t size;
+    if(!rhs.m_dynamic)
+      size = sizeof(typename Type::Data); //Create a dynamic data instance
+    else
+      size = rhs.dynamicSize(); //Create a constant data instance, that holds all the data embedded.
+
+    typename Type::Data& ret(*new (new char[size]) typename Type::Data(rhs));
+    ret.template setTypeClassId<Type>();
+    return ret;
+  }
+
+  /**
+   * As above, but does not support copying data into a lower class(Should not be used while cloning)
+   */
   template<class DataType>
-  static DataType& copyData(const DataType& rhs) {
+  static DataType& copyDataDirectly(const DataType& rhs) {
     size_t size;
     if(!rhs.m_dynamic)
       size = sizeof(DataType); //Create a dynamic data instance
@@ -228,21 +247,16 @@ public:
     return *new (new char[size]) DataType(rhs);
   }
 
-  template<class Type>
-  static typename Type::Data& copyDataDownwards(const typename Type::Data& rhs) {
-    typename Type::Data& ret(copyData<typename Type::Data>(rhs));
-    ret.template setTypeClassId<Type>();
-    return ret;
-  }
-  
   /**
    * Method to create internal data structures. Use this in normal constructors.
    *
    * \returns the internal data structure
    */
-  template<class DataType>
-  static DataType& createData() {
-    return *new (new char[sizeof(DataType)]) DataType();
+  template<class Type>
+  static typename Type::Data& createData() {
+    typename Type::Data& ret(*new (new char[sizeof(typename Type::Data)]) typename Type::Data());
+    ret.template setTypeClassId<Type>();
+    return ret;
   }
 
   typedef AbstractTypeData Data;
