@@ -69,15 +69,23 @@ struct FileModificationPairRequest {
   }
 
   void createItem(FileModificationPair* item) const {
-    *item = m_data;
+    new (item)  FileModificationPair(m_data);
   }
   
   bool equals(const FileModificationPair* item) const {
     return *item == m_data;
   }
+  
+  static void destroy(FileModificationPair* item, KDevelop::AbstractItemRepository&) {
+    item->~FileModificationPair();
+  }
+  
+  static bool persistent(const FileModificationPair* /*item*/) {
+    return true; //Reference-counting is done implicitly using the set-repository
+  }
 };
 
-typedef KDevelop::ItemRepository<FileModificationPair, FileModificationPairRequest, KDevelop::NoDynamicData, false> FileModificationPairRepository;
+typedef KDevelop::ItemRepository<FileModificationPair, FileModificationPairRequest, true, false> FileModificationPairRepository;
 
 static FileModificationPairRepository& fileModificationPairRepository() {
   static FileModificationPairRepository rep("file modification repository");
@@ -95,7 +103,7 @@ void ModificationRevisionSet::clearCache() {
 }
 
 struct FileModificationSetRepository : public Utils::BasicSetRepository {
-  FileModificationSetRepository() : Utils::BasicSetRepository("file modification sets") {
+  FileModificationSetRepository() : Utils::BasicSetRepository("file modification sets", &KDevelop::globalItemRepositoryRegistry(), true) {
   }
   virtual void itemRemovedFromSets(uint index) {
     fileModificationPairRepository().deleteItem(index);
@@ -110,9 +118,15 @@ struct FileModificationSetRepositoryRepresenter {
   }
 };
 
+
 ModificationRevisionSet::ModificationRevisionSet(unsigned int index) : m_index(index) {
 }
 
+uint ModificationRevisionSet::size() const {
+  Utils::Set set = Utils::Set(m_index, &fileModificationSetRepository);
+  return set.count();
+}
+
 void ModificationRevisionSet::clear() {
   QMutexLocker lock(&modificationRevisionSetMutex);
   
@@ -220,6 +234,28 @@ static bool nodeNeedsUpdate(uint index) {
   return result;
 }
 
+
+QString ModificationRevisionSet::toString() const
+{
+  QMutexLocker lock(&modificationRevisionSetMutex);
+  QString ret = "[";
+  Utils::Set set(m_index, &fileModificationSetRepository);
+  Utils::Set::Iterator it = set.iterator();
+  bool first = true;
+  while(it) {
+    if(!first)
+      ret += ", ";
+    first = false;
+    
+    const FileModificationPair* data = fileModificationPairRepository().itemFromIndex(*it);
+    ret += data->file.str() + ":" + data->revision.toString();
+    ++it;
+  }
+
+  ret += "]";
+  return ret;
+}
+
 bool ModificationRevisionSet::needsUpdate() const {
   QMutexLocker lock(&modificationRevisionSetMutex);
   
@@ -230,7 +266,7 @@ bool ModificationRevisionSet::needsUpdate() const {
     const FileModificationPair* data = fileModificationPairRepository().itemFromIndex(*it);
     ModificationRevision revision = KDevelop::ModificationRevision::revisionForFile( data->file );
     if( revision != data->revision ) {
-       kDebug( 9007 ) << "dependency" << data->file.str() << "has changed, stored stamp:" << data->revision << "new time:" << revision ;
+       kDebug() << "dependency" << data->file.str() << "has changed, stored stamp:" << data->revision << "new time:" << revision ;
        return true;
     }
     ++it;
