@@ -67,11 +67,13 @@ class ContextBrowserPlugin : public KDevelop::IPlugin, public KTextEditor::Smart
     
     virtual KDevelop::ContextMenuExtension contextMenuExtension(KDevelop::Context*);
 
+    QWidget* toolbarWidgetForMainWindow(QWidget* widgetInWindow);
+    
   Q_SIGNALS:
     void previousContextShortcut();
     void nextContextShortcut();
 
-  private slots:
+  public slots:
     void startDelayedBrowsing(KTextEditor::View* view);
     void stopDelayedBrowsing();
     
@@ -94,7 +96,11 @@ class ContextBrowserPlugin : public KDevelop::IPlugin, public KTextEditor::Smart
     void hideTooTip();
     void findUses();
     
+    void textInserted(KTextEditor::Document*, KTextEditor::Range);
+    
   private:
+    
+    virtual void createActionsForMainWindow(Sublime::MainWindow* /*window*/, QString& xmlFile, KActionCollection& actions);
     void switchUse(bool forward);
     void clearMouseHover();
     virtual void rangeDeleted (KTextEditor::SmartRange *range);
@@ -122,9 +128,11 @@ class ContextBrowserPlugin : public KDevelop::IPlugin, public KTextEditor::Smart
 
   private:
     //Unhighlights all currently highlighted declarations
-    void unHighlightAll();
+    void unHighlightAll(KTextEditor::View* selectView = 0);
     void showToolTip(KTextEditor::View* view, KTextEditor::Cursor position);
     QTimer* m_updateTimer;
+    //Must be locked before doing anything with m_backups, and _after_ the relevant smart-mutex has been locked
+    QMutex m_backupsMutex;
     //Contains the range, the old attribute, and the attribute it was replaced with
     QMap<KTextEditor::SmartRange*, QPair<KTextEditor::Attribute::Ptr,KTextEditor::Attribute::Ptr> > m_backups;
     QSet<KTextEditor::View*> m_updateViews;
@@ -146,6 +154,12 @@ class ContextBrowserPlugin : public KDevelop::IPlugin, public KTextEditor::Smart
     QPointer<QWidget> m_currentToolTip;
     IndexedDeclaration m_currentToolTipDeclaration;
     QAction* m_findUses;
+    
+    //This is just a marker, only used for comparison. Never expect it to be valid!
+    KTextEditor::Document* m_lastInsertionDocument;
+    KTextEditor::Cursor m_lastInsertionPos;
+    QSet<KTextEditor::View*> m_keepHighlightedDeclaration;
+    QList<QPointer<QWidget> > m_toolbarWidgets;
 };
 
 DUContext* contextAt(const SimpleCursor& position, TopDUContext* topContext);
