@@ -57,60 +57,12 @@
 #include <interfaces/contextmenuextension.h>
 #include <interfaces/iplugincontroller.h>
 #include <ktexteditor/codecompletioninterface.h>
+#include <language/interfaces/iquickopen.h>
 
 const int maxHistoryLength = 30;
 
 using namespace KDevelop;
 
-QToolButton* ContextController::previousButton() const {
-    return m_previousButton;
-}
-
-QToolButton* ContextController::nextButton() const {
-    return m_nextButton;
-}
-
-QToolButton* ContextController::browseButton() const {
-    return m_browseButton;
-}
-
-ContextController::ContextController(ContextBrowserView* view) : QObject(view), m_nextHistoryIndex(0), m_view(view) {
-    m_browseManager = new BrowseManager(this);
-    
-    connect(ICore::self()->documentController(), SIGNAL(documentJumpPerformed(KDevelop::IDocument*, KTextEditor::Cursor, KDevelop::IDocument*, KTextEditor::Cursor)), this, SLOT(documentJumpPerformed(KDevelop::IDocument*, KTextEditor::Cursor, KDevelop::IDocument*, KTextEditor::Cursor)));
-    
-    connect(m_browseManager, SIGNAL(shiftKeyTriggered()), this, SLOT(switchFocusToContextBrowser()));
-    
-    m_previousButton = new QToolButton();
-    m_previousButton->setPopupMode(QToolButton::MenuButtonPopup);
-    m_previousButton->setIcon(KIcon("go-previous"));
-    m_previousButton->setEnabled(false);
-    m_previousMenu = new QMenu();
-    m_previousButton->setMenu(m_previousMenu);
-    connect(m_previousButton, SIGNAL(clicked(bool)), this, SLOT(historyPrevious()));
-    connect(m_previousMenu, SIGNAL(aboutToShow()), this, SLOT(previousMenuAboutToShow()));
-
-    m_nextButton = new QToolButton();
-    m_nextButton->setPopupMode(QToolButton::MenuButtonPopup);
-    m_nextButton->setIcon(KIcon("go-next"));
-    m_nextButton->setEnabled(false);
-    m_nextMenu = new QMenu();
-    m_nextButton->setMenu(m_nextMenu);
-    connect(m_nextButton, SIGNAL(clicked(bool)), this, SLOT(historyNext()));
-    connect(m_nextMenu, SIGNAL(aboutToShow()), this, SLOT(nextMenuAboutToShow()));
-
-    m_browseButton = new QToolButton();
-    m_browseButton->setIcon(KIcon("games-hint"));
-    m_browseButton->setToolTip(i18n("Enable/disable source browse mode"));
-    m_browseButton->setWhatsThis(i18n("When this is enabled, you can browse the source-code by clicking in the editor."));
-    m_browseButton->setCheckable(true);
-    connect(m_browseButton, SIGNAL(clicked(bool)), m_browseManager, SLOT(setBrowsing(bool)));
-
-    m_currentContextBox = new KComboBox();
-    m_currentContextBox->setSizeAdjustPolicy(QComboBox::AdjustToContents);
-    connect(m_currentContextBox, SIGNAL(activated(int)), this, SLOT(comboItemActivated(int)));
-}
-
 ///Duchain must be locked
 DUContext* getContextAt(KUrl url, KTextEditor::Cursor cursor) {
     TopDUContext* topContext = DUChainUtils::standardContextForUrl(url);
@@ -179,13 +131,12 @@ void ContextBrowserView::navigateUp() {
         widget->up();
 }
 
-void ContextController::documentJumpPerformed( KDevelop::IDocument* newDocument, KTextEditor::Cursor newCursor, KDevelop::IDocument* previousDocument, KTextEditor::Cursor previousCursor) {
+void ContextBrowserView::documentJumpPerformed( KDevelop::IDocument* newDocument, KTextEditor::Cursor newCursor, KDevelop::IDocument* previousDocument, KTextEditor::Cursor previousCursor) {
         
-    kDebug() << "jump from" << (previousDocument ? previousDocument->url().prettyUrl() : QString()) << "to" << (newDocument ? newDocument->url().prettyUrl() : QString());
-    
     DUChainReadLocker lock(DUChain::lock());
     
     if(previousDocument && previousCursor.isValid()) {
+        kDebug() << "updating jump source";
         DUContext* context = getContextAt(previousDocument->url(), previousCursor);
         if(context) {
             updateHistory(context, SimpleCursor(previousCursor), true);
@@ -196,7 +147,9 @@ void ContextController::documentJumpPerformed( KDevelop::IDocument* newDocument,
             ++m_nextHistoryIndex;
         }
     }
+    kDebug() << "new doc: " << newDocument << " new cursor: " << newCursor;
     if(newDocument && newCursor.isValid()) {
+        kDebug() << "updating jump target";
         DUContext* context = getContextAt(newDocument->url(), newCursor);
         if(context) {
             updateHistory(context, SimpleCursor(newCursor), true);
@@ -205,64 +158,29 @@ void ContextController::documentJumpPerformed( KDevelop::IDocument* newDocument,
             m_history.resize(m_nextHistoryIndex); // discard forward history
             m_history.append(HistoryEntry(DocumentCursor(newDocument->url().prettyUrl(), newCursor)));
             ++m_nextHistoryIndex;
+            m_outlineLine->clear();
         }
     }
 }
 
-ContextBrowserView* ContextController::view() const {
-    return m_view;
-}
-
-KComboBox* ContextController::currentContextBox() const {
-    return m_currentContextBox;
-}
-
-ContextController::~ContextController() {
-    delete m_nextMenu;
-    delete m_previousMenu;
-}
-
-void ContextController::updateButtonState()
+void ContextBrowserView::updateButtonState()
 {
     m_nextButton->setEnabled( m_nextHistoryIndex < m_history.size() );
     m_previousButton->setEnabled( m_nextHistoryIndex >= 2 );
 }
 
-void ContextController::comboItemActivated(int index)
-{
-    if(index >= 0 && index < m_listDeclarations.size()) {
-        IndexedString u;
-        SimpleCursor c;
-        {
-            KDevelop::DUChainReadLocker lock( KDevelop::DUChain::lock() );
-            Declaration* activated = m_listDeclarations[index].data();
-            if(activated) {
-                u = activated->url();
-                c = activated->range().start;
-                if(activated->internalContext() && activated->internalContext()->url() == u) {
-                    c = activated->internalContext()->range().start;
-                    if(c.line+1 <= activated->internalContext()->range().end.line)
-                        c = SimpleCursor(c.line+1, 0); //Move more into the body
-                }
-            }
-        }
-        if(c.isValid())
-            ICore::self()->documentController()->openDocument(KUrl(u.str()), c.textCursor());
-    }
-}
-
-void ContextController::historyNext() {
+void ContextBrowserView::historyNext() {
     if(m_nextHistoryIndex >= m_history.size()) {
         return;
     }
-    m_view->allowLockedUpdate();
+    allowLockedUpdate();
     openDocument(m_nextHistoryIndex); // opening the document at given position 
                                       // will update the widget for us
     ++m_nextHistoryIndex;
     updateButtonState();
 }
 
-void ContextController::openDocument(int historyIndex) {
+void ContextBrowserView::openDocument(int historyIndex) {
     Q_ASSERT_X(historyIndex >= 0, "openDocument", "negative history index");
     Q_ASSERT_X(historyIndex < m_history.size(), "openDocument", "history index out of range");
     DocumentCursor c = m_history[historyIndex].computePosition();
@@ -279,18 +197,18 @@ void ContextController::openDocument(int historyIndex) {
     }
 }
 
-void ContextController::historyPrevious() {
+void ContextBrowserView::historyPrevious() {
     if(m_nextHistoryIndex < 2) {
         return;
     }
     --m_nextHistoryIndex;
-    m_view->allowLockedUpdate();
+    allowLockedUpdate();
     openDocument(m_nextHistoryIndex-1); // opening the document at given position 
                                         // will update the widget for us
     updateButtonState();
 }
 
-QString ContextController::actionTextFor(int historyIndex)
+QString ContextBrowserView::actionTextFor(int historyIndex)
 {
     HistoryEntry& entry = m_history[historyIndex];
     QString actionText = entry.context.data() ? entry.context.data()->scopeIdentifier(true).toString() : QString();
@@ -304,14 +222,14 @@ QString ContextController::actionTextFor(int historyIndex)
     return actionText;
 }
 
-inline QDebug operator<<(QDebug debug, const ContextController::HistoryEntry &he)
+inline QDebug operator<<(QDebug debug, const ContextBrowserView::HistoryEntry &he)
 {
     DocumentCursor c = he.computePosition();
     debug << "\n\tHistoryEntry " << c.line() << " " << c.document().str();
     return debug;
 }
 
-void ContextController::nextMenuAboutToShow() {
+void ContextBrowserView::nextMenuAboutToShow() {
     QList<int> indices;
     for(int a = m_nextHistoryIndex; a < m_history.size(); ++a) {
         indices << a;
@@ -319,7 +237,7 @@ void ContextController::nextMenuAboutToShow() {
     fillHistoryPopup(m_nextMenu, indices);
 }
 
-void ContextController::previousMenuAboutToShow() {
+void ContextBrowserView::previousMenuAboutToShow() {
     QList<int> indices;
     for(int a = m_nextHistoryIndex-2; a >= 0; --a) {
         indices << a;
@@ -327,7 +245,7 @@ void ContextController::previousMenuAboutToShow() {
     fillHistoryPopup(m_previousMenu, indices);
 }
 
-void ContextController::fillHistoryPopup(QMenu* menu, const QList<int>& historyIndices) {
+void ContextBrowserView::fillHistoryPopup(QMenu* menu, const QList<int>& historyIndices) {
     menu->clear();
     KDevelop::DUChainReadLocker lock( KDevelop::DUChain::lock() );
     foreach(int index, historyIndices) {
@@ -338,37 +256,33 @@ void ContextController::fillHistoryPopup(QMenu* menu, const QList<int>& historyI
     }
 }
 
-QWidget* ContextController::focusBackWidget() {
-    return m_focusBackWidget;
-}
-
-void ContextController::switchFocusToContextBrowser() {
-    if(m_view->isVisible()) {
+void ContextBrowserView::switchFocusToContextBrowser() {
+    if(isVisible()) {
         kDebug() << "switching focus to context-browser";
-        if(QApplication::focusWidget() != m_view)
+        if(QApplication::focusWidget() != this)
             m_focusBackWidget = QApplication::focusWidget();
-        m_view->setFocus();
+        setFocus();
     }
 }
 
-void ContextController::actionTriggered() {
+void ContextBrowserView::actionTriggered() {
     QAction* action = qobject_cast<QAction*>(sender());
     Q_ASSERT(action); Q_ASSERT(action->data().type() == QVariant::Int);
     int historyPosition = action->data().toInt();
     // kDebug() << "history pos" << historyPosition << m_history.size() << m_history;
     if(historyPosition >= 0 && historyPosition < m_history.size()) {
         m_nextHistoryIndex = historyPosition + 1;
-        m_view->allowLockedUpdate(); // opening the document at given position 
+        allowLockedUpdate(); // opening the document at given position 
                                      // will update the widget for us
         openDocument(historyPosition);
         updateButtonState();
     }
 }
 
-ContextController::HistoryEntry::HistoryEntry(KDevelop::DocumentCursor pos) : absoluteCursorPosition(pos) {
+ContextBrowserView::HistoryEntry::HistoryEntry(KDevelop::DocumentCursor pos) : absoluteCursorPosition(pos) {
 }
 
-ContextController::HistoryEntry::HistoryEntry(IndexedDUContext ctx, const KDevelop::SimpleCursor& cursorPosition) : context(ctx) {
+ContextBrowserView::HistoryEntry::HistoryEntry(IndexedDUContext ctx, const KDevelop::SimpleCursor& cursorPosition) : context(ctx) {
         //Use a position relative to the context
         setCursorPosition(cursorPosition);
         if(ctx.data())
@@ -377,7 +291,7 @@ ContextController::HistoryEntry::HistoryEntry(IndexedDUContext ctx, const KDevel
             alternativeString += i18n("(changed)"); //This is used when the context was deleted in between
 }
 
-DocumentCursor ContextController::HistoryEntry::computePosition() const {
+DocumentCursor ContextBrowserView::HistoryEntry::computePosition() const {
     KDevelop::DUChainReadLocker lock( KDevelop::DUChain::lock() );
     DocumentCursor ret;
     if(context.data()) {
@@ -389,7 +303,7 @@ DocumentCursor ContextController::HistoryEntry::computePosition() const {
     return ret;
 }
 
-void ContextController::HistoryEntry::setCursorPosition(const KDevelop::SimpleCursor& cursorPosition) {
+void ContextBrowserView::HistoryEntry::setCursorPosition(const KDevelop::SimpleCursor& cursorPosition) {
     KDevelop::DUChainReadLocker lock( KDevelop::DUChain::lock() );
     if(context.data()) {
         absoluteCursorPosition =  DocumentCursor(context.data()->url().str(), cursorPosition.textCursor());
@@ -398,7 +312,7 @@ void ContextController::HistoryEntry::setCursorPosition(const KDevelop::SimpleCu
     }
 }
 
-bool ContextController::isPreviousEntry(KDevelop::DUContext* context, const KDevelop::SimpleCursor& /*position*/) {
+bool ContextBrowserView::isPreviousEntry(KDevelop::DUContext* context, const KDevelop::SimpleCursor& /*position*/) {
     if (m_nextHistoryIndex == 0) return false;
     Q_ASSERT(m_nextHistoryIndex <= m_history.count());
     HistoryEntry& he = m_history[m_nextHistoryIndex-1];
@@ -407,62 +321,42 @@ bool ContextController::isPreviousEntry(KDevelop::DUContext* context, const KDev
     return IndexedDUContext(context) == he.context;
 }
 
-void ContextController::updateDeclarationListBox(DUContext* context) {
-    if(!context) {
+void ContextBrowserView::updateDeclarationListBox(DUContext* context) {
+    if(!context || !context->owner()) {
+        kDebug() << "not updating box";
         m_listUrl = IndexedString(); ///@todo Compute the context in the document here
-        m_currentContextBox->clear();
+        m_outlineLine->clear();
         return;
     }
+    
+    Declaration* decl = context->owner();
+    
     m_listUrl = context->url();
 
-    class FunctionListFilter : public DUChainUtils::DUChainItemFilter {
-        public:
-        FunctionListFilter(KComboBox* box, QList<IndexedDeclaration>& _declarations) : m_box(box), declarations(_declarations) {
-            box->clear();
-            declarations.clear();
-        }
-        
-        virtual bool accept(Declaration* decl) {
-            if(decl->range().isEmpty())
-                return false;
-                
-            if(decl->isFunctionDeclaration() || (decl->internalContext() && decl->internalContext()->type() == DUContext::Class)) {
-                Declaration* specialDecl = SpecializationStore::self().applySpecialization(decl, decl->topContext());
+    Declaration* specialDecl = SpecializationStore::self().applySpecialization(decl, decl->topContext());
 
-                FunctionType::Ptr function = specialDecl->type<FunctionType>();
-                QString text = specialDecl->qualifiedIdentifier().toString();
-                if(function)
-                    text += function->partToString(KDevelop::FunctionType::SignatureArguments);
-
-                m_box->addItem(text);
-                declarations.append(decl);
-            }
-
-            return true;
-        }
-        virtual bool accept(DUContext* ctx) {
-        return ctx->type() == DUContext::Global || ctx->type() == DUContext::Namespace || ctx->type() == DUContext::Class;
-        }
-        KComboBox* m_box;
-        QList<IndexedDeclaration>& declarations;
-    };
-
-    m_currentContextBox->clear();
-    FunctionListFilter f(currentContextBox(), m_listDeclarations);
-    DUChainUtils::collectItems(context->topContext(), f);
-    currentContextBox()->setCurrentIndex(m_listDeclarations.indexOf(context->owner()));
+    FunctionType::Ptr function = specialDecl->type<FunctionType>();
+    QString text = specialDecl->qualifiedIdentifier().toString();
+    if(function)
+        text += function->partToString(KDevelop::FunctionType::SignatureArguments);
+    
+    m_outlineLine->setText(text);
+    m_outlineLine->setCursorPosition(0);
+    
+    kDebug() << "updated" << text;
 }
 
-void ContextController::updateHistory(KDevelop::DUContext* context, const KDevelop::SimpleCursor& position, bool force)
+void ContextBrowserView::updateHistory(KDevelop::DUContext* context, const KDevelop::SimpleCursor& position, bool force)
 {
-    if (context == 0) return;
-    if(!context->owner() && !force)
+    kDebug() << "updating history";
+    
+    if(m_outlineLine->isVisible())
+        updateDeclarationListBox(context);
+    
+    if(!context && !context->owner() && !force) {
         return; //Only add history-entries for contexts that have owners, which in practice should be functions and classes
                 //This keeps the history cleaner
-
-    if (!isPreviousEntry(context, position) || context->url() != m_listUrl)
-        if(m_currentContextBox->isVisible())
-            updateDeclarationListBox(context);
+    }
 
     if (isPreviousEntry(context, position)) {
         if(m_nextHistoryIndex) {
@@ -483,7 +377,7 @@ void ContextController::updateHistory(KDevelop::DUContext* context, const KDevel
     }
 }
 
-QWidget* ContextController::createWidget(KDevelop::DUContext* context) {
+QWidget* ContextBrowserView::createWidget(KDevelop::DUContext* context) {
         m_context = IndexedDUContext(context);
         if(m_context.data()) {
             return m_context.data()->createNavigationWidget();
@@ -491,14 +385,11 @@ QWidget* ContextController::createWidget(KDevelop::DUContext* context) {
         return 0;
 }
 
-DeclarationController::DeclarationController() {
-}
-
-KDevelop::IndexedDeclaration DeclarationController::declaration() {
+KDevelop::IndexedDeclaration ContextBrowserView::declaration() const {
     return m_declaration;
 }
 
-QWidget* DeclarationController::createWidget(Declaration* decl, TopDUContext* topContext) {
+QWidget* ContextBrowserView::createWidget(Declaration* decl, TopDUContext* topContext) {
     m_declaration = IndexedDeclaration(decl);
     return decl->context()->createNavigationWidget(decl, topContext);
 }
@@ -534,14 +425,53 @@ void ContextBrowserView::updateLockIcon(bool checked) {
     m_lockButton->setIcon(KIcon(checked ? "document-encrypt" : "document-decrypt"));
 }
 
-ContextBrowserView::ContextBrowserView( ContextBrowserPlugin* plugin ) : m_plugin(plugin), m_navigationWidget(new KTextBrowser()) {
+ContextBrowserView::ContextBrowserView( ContextBrowserPlugin* plugin, QWidget* parent ) : QWidget(parent), m_plugin(plugin), m_navigationWidget(new KTextBrowser()), m_nextHistoryIndex(0), m_outlineLine(0) {
     setWindowIcon( KIcon("applications-development-web") );
 
-    m_declarationCtrl = new DeclarationController();
-    m_contextCtrl = new ContextController(this);
+    m_browseManager = new BrowseManager(this);
     
-    connect(m_contextCtrl->browseManager(), SIGNAL(startDelayedBrowsing(KTextEditor::View*)), this, SIGNAL(startDelayedBrowsing(KTextEditor::View*)));
-    connect(m_contextCtrl->browseManager(), SIGNAL(stopDelayedBrowsing()), this, SIGNAL(stopDelayedBrowsing()));
+    connect(ICore::self()->documentController(), SIGNAL(documentJumpPerformed(KDevelop::IDocument*, KTextEditor::Cursor, KDevelop::IDocument*, KTextEditor::Cursor)), this, SLOT(documentJumpPerformed(KDevelop::IDocument*, KTextEditor::Cursor, KDevelop::IDocument*, KTextEditor::Cursor)));
+    
+    connect(m_browseManager, SIGNAL(shiftKeyTriggered()), this, SLOT(switchFocusToContextBrowser()));
+    
+    m_previousButton = new QToolButton();
+    m_previousButton->setPopupMode(QToolButton::MenuButtonPopup);
+    m_previousButton->setIcon(KIcon("go-previous"));
+    m_previousButton->setEnabled(false);
+    m_previousButton->setFocusPolicy(Qt::NoFocus);
+    m_previousMenu = new QMenu();
+    m_previousButton->setMenu(m_previousMenu);
+    connect(m_previousButton, SIGNAL(clicked(bool)), this, SLOT(historyPrevious()));
+    connect(m_previousMenu, SIGNAL(aboutToShow()), this, SLOT(previousMenuAboutToShow()));
+
+    m_nextButton = new QToolButton();
+    m_nextButton->setPopupMode(QToolButton::MenuButtonPopup);
+    m_nextButton->setIcon(KIcon("go-next"));
+    m_nextButton->setEnabled(false);
+    m_nextButton->setFocusPolicy(Qt::NoFocus);
+    m_nextMenu = new QMenu();
+    m_nextButton->setMenu(m_nextMenu);
+    connect(m_nextButton, SIGNAL(clicked(bool)), this, SLOT(historyNext()));
+    connect(m_nextMenu, SIGNAL(aboutToShow()), this, SLOT(nextMenuAboutToShow()));
+
+    m_browseButton = new QToolButton();
+    m_browseButton->setIcon(KIcon("games-hint"));
+    m_browseButton->setToolTip(i18n("Enable/disable source browse mode"));
+    m_browseButton->setWhatsThis(i18n("When this is enabled, you can browse the source-code by clicking in the editor."));
+    m_browseButton->setCheckable(true);
+    m_browseButton->setFocusPolicy(Qt::NoFocus);
+    
+    connect(m_browseButton, SIGNAL(clicked(bool)), m_browseManager, SLOT(setBrowsing(bool)));
+
+    IQuickOpen* quickOpen = KDevelop::ICore::self()->pluginController()->extensionForPlugin<IQuickOpen>("org.kdevelop.IQuickOpen");
+    
+    if(quickOpen) {
+      m_outlineLine = quickOpen->createQuickOpenLine(QStringList(), QStringList() << i18n("Outline"), IQuickOpen::Outline);
+      m_outlineLine->setDefaultText(i18n("Outline..."));
+    }
+    
+    connect(m_browseManager, SIGNAL(startDelayedBrowsing(KTextEditor::View*)), this, SIGNAL(startDelayedBrowsing(KTextEditor::View*)));
+    connect(m_browseManager, SIGNAL(stopDelayedBrowsing()), this, SIGNAL(stopDelayedBrowsing()));
     
     m_allowLockedUpdate = false;
     
@@ -554,14 +484,25 @@ ContextBrowserView::ContextBrowserView( ContextBrowserPlugin* plugin ) : m_plugi
     connect(m_lockButton, SIGNAL(toggled(bool)), SLOT(updateLockIcon(bool)));
 
     m_declarationMenuButton = new QToolButton();
-    m_declarationMenuButton->setIcon(KIcon("class"));
+    m_declarationMenuButton->setIcon(KIcon("code-class"));
     m_declarationMenuButton->setToolTip(i18n("Declaration menu"));
     connect(m_declarationMenuButton, SIGNAL(clicked(bool)), SLOT(declarationMenu()));
     
-    m_buttons->addWidget(m_contextCtrl->previousButton());
-    m_buttons->addWidget(m_contextCtrl->currentContextBox());
-    m_buttons->addWidget(m_contextCtrl->nextButton());
-    m_buttons->addWidget(m_contextCtrl->browseButton());
+    m_toolbarWidget = plugin->toolbarWidgetForMainWindow(this);
+    m_toolbarWidgetLayout = new QHBoxLayout;
+    m_toolbarWidgetLayout->setSizeConstraint(QLayout::SetMaximumSize);
+    m_previousButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
+    m_nextButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
+    m_browseButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
+    
+    m_toolbarWidgetLayout->addWidget(m_previousButton);
+    m_toolbarWidgetLayout->addWidget(m_outlineLine);
+    m_toolbarWidgetLayout->addWidget(m_nextButton);
+    m_toolbarWidgetLayout->addWidget(m_browseButton);
+    
+    if(m_toolbarWidget->children().isEmpty())
+        m_toolbarWidget->setLayout(m_toolbarWidgetLayout);
+    
     m_buttons->addStretch();
     m_buttons->addWidget(m_declarationMenuButton);
     m_buttons->addWidget(m_lockButton);
@@ -573,12 +514,25 @@ ContextBrowserView::ContextBrowserView( ContextBrowserPlugin* plugin ) : m_plugi
     setLayout(m_layout);
 
     m_plugin->registerToolView(this);
-    connect(plugin, SIGNAL(previousContextShortcut()), m_contextCtrl, SLOT(historyPrevious()));
-    connect(plugin, SIGNAL(nextContextShortcut()), m_contextCtrl, SLOT(historyNext()));
+    
+    connect(plugin, SIGNAL(previousContextShortcut()), this, SLOT(historyPrevious()));
+    connect(plugin, SIGNAL(nextContextShortcut()), this, SLOT(historyNext()));
+    
+    connect(ICore::self()->documentController(), SIGNAL(documentClosed(KDevelop::IDocument*)), m_outlineLine, SLOT(clear()));
+    connect(ICore::self()->documentController(), SIGNAL(documentActivated(KDevelop::IDocument*)), m_outlineLine, SLOT(clear()));
 }
 
 ContextBrowserView::~ContextBrowserView() {
     m_plugin->unRegisterToolView(this);
+    delete m_nextMenu;
+    delete m_previousMenu;
+    delete m_toolbarWidgetLayout;
+    
+    delete m_previousButton;
+    delete m_outlineLine;
+    delete m_nextButton;
+    delete m_browseButton;
+    
 }
 
 void ContextBrowserView::focusInEvent(QFocusEvent* event) {
@@ -621,29 +575,11 @@ bool ContextBrowserView::event(QEvent* event) {
             
             if(key == Qt::Key_L)
                 m_lockButton->toggle();
-            if(key == Qt::Key_F) {
-                m_contextCtrl->currentContextBox()->setFocus();
-                m_contextCtrl->currentContextBox()->removeEventFilter(this); //Just to prevent double insertion
-                m_contextCtrl->currentContextBox()->installEventFilter(this);
-            }
         }
     }
     return QWidget::event(event);
 }
 
-bool ContextBrowserView::eventFilter(QObject* object, QEvent* event) {
-    if(object == m_contextCtrl->currentContextBox() && m_contextCtrl->currentContextBox()->hasFocus()) {
-        QKeyEvent* keyEvent = dynamic_cast<QKeyEvent*>(event);
-        if(keyEvent && keyEvent->type() == QEvent::KeyPress && keyEvent->key() == Qt::Key_F) {
-            m_contextCtrl->currentContextBox()->removeEventFilter(this);
-            setFocus();
-        }
-    }
-    return QWidget::eventFilter(object, event);
-}
-
-
-
 void ContextBrowserView::showEvent(QShowEvent* event) {
     DUChainReadLocker lock(DUChain::lock());
     TopDUContext* top = m_lastUsedTopContext.data();
@@ -659,7 +595,7 @@ void ContextBrowserView::showEvent(QShowEvent* event) {
                 KTextEditor::Cursor c = doc->textDocument()->activeView()->cursorPosition();
                 context = getContextAt(top->url().toUrl(), c);
             }
-            m_contextCtrl->updateDeclarationListBox(context);
+            updateDeclarationListBox(context);
         }
     }
     QWidget::showEvent(event);
@@ -675,13 +611,6 @@ bool ContextBrowserView::isLocked() const {
     return isLocked;
 }
 
-void ContextBrowserView::updateHistory(KDevelop::DUContext* context, const KDevelop::SimpleCursor& position)
-{
-    if (!isLocked()) {
-        m_contextCtrl->updateHistory(context, position);
-    }
-}
-
 void ContextBrowserView::updateMainWidget(QWidget* widget)
 {
     if (widget) {
@@ -704,14 +633,14 @@ void ContextBrowserView::setDeclaration(KDevelop::Declaration* decl, KDevelop::T
     
     if (!isLocked() && (isVisible() || force)) {  // NO-OP if toolview is hidden, for performance reasons
         
-        QWidget* w = m_declarationCtrl->createWidget(decl, topContext);
+        QWidget* w = createWidget(decl, topContext);
         updateMainWidget(w);
     }
 }
 
 KDevelop::IndexedDeclaration ContextBrowserView::lockedDeclaration() const {
     if(m_lockButton->isChecked())
-        return m_declarationCtrl->declaration();
+        return declaration();
     else
         return KDevelop::IndexedDeclaration();
 }
@@ -740,7 +669,7 @@ void ContextBrowserView::setContext(KDevelop::DUContext* context) {
     
     if (!isLocked() && isVisible()) { // NO-OP if toolview is hidden, for performance reasons
         
-        QWidget* w = m_contextCtrl->createWidget(context);
+        QWidget* w = createWidget(context);
         updateMainWidget(w);
     }
 }
@@ -752,8 +681,8 @@ void ContextBrowserView::setSpecialNavigationWidget(QWidget* widget) {
     }
 }
 
-BrowseManager* ContextController::browseManager() const {
-    return m_browseManager;
+void ContextBrowserView::setAllowBrowsing(bool allow) {
+    m_browseButton->setChecked(allow);
 }
 
 
