@@ -37,6 +37,7 @@
 #include <sublime/document.h>
 
 #include "documentswitchertreeview.h"
+#include <QStandardItemModel>
 
 K_PLUGIN_FACTORY(DocumentSwitcherFactory, registerPlugin<DocumentSwitcherPlugin>(); )
 K_EXPORT_PLUGIN(DocumentSwitcherFactory(KAboutData("kdevdocumentswitcher","kdevdocumentswitcher",ki18n("Document Switcher"), "0.1", ki18n("Switch between open documents using most-recently-used list"), KAboutData::License_GPL)))
@@ -74,7 +75,7 @@ DocumentSwitcherPlugin::DocumentSwitcherPlugin(QObject *parent, const QVariantLi
     connect( view, SIGNAL(clicked(const QModelIndex&)), SLOT(switchToView(const QModelIndex&)) );
     connect( view, SIGNAL(activated(const QModelIndex&)), SLOT(switchToView(const QModelIndex&)) );
     
-    model = new QStringListModel( view );
+    model = new QStandardItemModel( view );
     view->setModel( model );    
 }
 
@@ -89,15 +90,15 @@ void DocumentSwitcherPlugin::walkForward()
     QModelIndex idx;
     if( !view->isVisible() )
     {
-        QStringList views;
-        foreach( Sublime::View* v, documentLists[window][window->area()] )
-        {
-            views << v->document()->title();
-        }
-    
-        model->setStringList( views );
-        view->move( QCursor::pos() );
+        fillModel( window );
+        // center on main window
+        view->move( window->pos().x() + (window->width() - view->width()) / 2,
+                    window->pos().y() + (window->height() - view->height()) / 2 );
         idx = model->index( 1, 0 );
+        if( !idx.isValid() )
+        {
+            idx = model->index( 0, 0 );
+        }
         view->show();
     } else 
     {
@@ -108,8 +109,20 @@ void DocumentSwitcherPlugin::walkForward()
         }
         idx = model->index( newrow, 0 );
     }
-    view->selectionModel()->select( idx, QItemSelectionModel::ClearAndSelect );
-    view->selectionModel()->setCurrentIndex( idx, QItemSelectionModel::SelectCurrent );
+    view->selectionModel()->select( idx, QItemSelectionModel::Rows | QItemSelectionModel::ClearAndSelect );
+    view->selectionModel()->setCurrentIndex( idx, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Rows );
+}
+
+
+void DocumentSwitcherPlugin::fillModel( Sublime::MainWindow* window )
+{
+    model->clear();
+    QList<QStandardItem*> views;
+    foreach( Sublime::View* v, documentLists[window][window->area()] )
+    {   
+        model->appendRow( new QStandardItem( v->document()->statusIcon(), v->document()->title() ) );
+    }
+    
 }
 
 void DocumentSwitcherPlugin::walkBackward()
@@ -123,14 +136,10 @@ void DocumentSwitcherPlugin::walkBackward()
     QModelIndex idx;
     if( !view->isVisible() )
     {
-        QStringList views;
-        foreach( Sublime::View* v, documentLists[window][window->area()] )
-        {
-            views << v->document()->title();
-        }
-    
-        model->setStringList( views );
-        view->move( QCursor::pos() );
+        fillModel( window );
+        // center on mainwindow
+        view->move( window->pos().x() + (window->width() - view->width()) / 2,
+                    window->pos().y() + (window->height() - view->height()) / 2 );
         idx = model->index( model->rowCount()-1, 0 );
         view->show();
     } else 
@@ -142,8 +151,8 @@ void DocumentSwitcherPlugin::walkBackward()
         }
         idx = model->index( newrow, 0 );
     }
-    view->selectionModel()->select( idx, QItemSelectionModel::ClearAndSelect );
-    view->selectionModel()->setCurrentIndex( idx, QItemSelectionModel::SelectCurrent );
+    view->selectionModel()->select( idx, QItemSelectionModel::Rows | QItemSelectionModel::ClearAndSelect );
+    view->selectionModel()->setCurrentIndex( idx, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Rows );
 }
 
 DocumentSwitcherPlugin::~DocumentSwitcherPlugin()
@@ -152,8 +161,13 @@ DocumentSwitcherPlugin::~DocumentSwitcherPlugin()
 
 void DocumentSwitcherPlugin::switchToView( const QModelIndex& idx )
 {
-    int row = view->selectionModel()->selectedRows().first().row();
+    Q_UNUSED( idx );
     view->hide();
+    if( view->selectionModel()->selectedRows().isEmpty() )
+    {
+        return;
+    }
+    int row = view->selectionModel()->selectedRows().first().row();
     
     Sublime::MainWindow* window = qobject_cast<Sublime::MainWindow*>( KDevelop::ICore::self()->uiController()->activeMainWindow() );
     if( window && documentLists.contains( window ) && documentLists[window].contains( window->area() ) )
@@ -168,6 +182,10 @@ void DocumentSwitcherPlugin::switchToView( const QModelIndex& idx )
 
 void DocumentSwitcherPlugin::unload()
 {
+    foreach( QObject* mw, documentLists.keys() )
+    {
+        removeMainWindow( mw );
+    }
     delete forwardAction;
     delete backwardAction;
     view->deleteLater();
@@ -227,12 +245,12 @@ void DocumentSwitcherPlugin::enableActions( Sublime::MainWindow* mw )
 
 void DocumentSwitcherPlugin::removeMainWindow( QObject* obj ) 
 {
-    Sublime::MainWindow* mainwindow = qobject_cast<Sublime::MainWindow*>( obj );
-    if( !mainwindow )
+    if( !obj || !documentLists.contains(obj) ) {
         return;
-    mainwindow->removeEventFilter( this );
-    disconnect( mainwindow, 0, this, 0 );
-    documentLists.remove( mainwindow );
+    }
+    obj->removeEventFilter( this );
+    disconnect( obj, 0, this, 0 );
+    documentLists.remove( obj );
 }
 
 
