@@ -44,6 +44,11 @@
 #include <KIO/Job>
 #include <KIO/NetAccess>
 
+///FIXME: remove the hack and these includes
+#include <interfaces/isession.h>
+#include <language/backgroundparser/parseprojectjob.h>
+///END
+
 K_PLUGIN_FACTORY(GenericSupportFactory, registerPlugin<GenericProjectManager>(); )
 K_EXPORT_PLUGIN(GenericSupportFactory(KAboutData("kdevgenericmanager","kdevgenericprojectmanager",ki18n("Generic Project Manager"), "0.1", ki18n("A plugin to support basic project management on a filesystem level"), KAboutData::License_GPL)))
 
@@ -126,13 +131,39 @@ void GenericProjectManager::waitForProjectOpen(KDevelop::IProject* project)
 void GenericProjectManager::eventuallyReadFolder( KDevelop::ProjectFolderItem* item )
 {
     ///TODO: listRecursive does not follow directory links!
-    KIO::ListJob* job = KIO::listRecursive( item->url() );
+    KIO::ListJob* job = KIO::listRecursive( item->url(), KIO::HideProgressInfo );
     kDebug() << "adding job" << job << job->url() << "for project" << item->project();
 
     KDevelop::ICore::self()->runController()->registerJob( job );
 
     connect( job, SIGNAL(entries(KIO::Job*, KIO::UDSEntryList)),
                 this, SLOT(addJobItems(KIO::Job*, KIO::UDSEntryList)) );
+
+    /// FIXME: once async project managers are supported, we won't have to do this hack
+    /// since parse() cannot return anything the parseprojectjob gets a list of empty files
+    /// => hence this hack at least starts the parsepojrectjob if required
+    if ( item->isProjectRoot() &&
+         KDevelop::ICore::self()->activeSession()->config()->group( "Project Manager" ).readEntry( "Parse All Project Sources", true ) )
+    {
+        connect( job, SIGNAL(result(KJob*)),
+                this, SLOT(startProjectParseJob(KJob*)) );
+    }
+}
+
+void GenericProjectManager::startProjectParseJob( KJob* j )
+{
+    if ( j->error() ) {
+        return;
+    }
+    KIO::SimpleJob* job(dynamic_cast<KIO::SimpleJob*>(j));
+    Q_ASSERT(job);
+
+    KDevelop::IProject *project = KDevelop::ICore::self()->projectController()->findProjectForUrl(job->url());
+    if ( !project ) {
+        return;
+    }
+    KJob* parseProjectJob = new KDevelop::ParseProjectJob(project);
+    KDevelop::ICore::self()->runController()->registerJob(parseProjectJob);
 }
 
 void GenericProjectManager::addJobItems(KIO::Job* j, KIO::UDSEntryList entries)
@@ -153,7 +184,10 @@ void GenericProjectManager::addJobItems(KIO::Job* j, KIO::UDSEntryList entries)
     kDebug() << "reading contents of " << curUrl << "total entries:" << entries.count();
 
     KDevelop::IProject* project = KDevelop::ICore::self()->projectController()->findProjectForUrl( job->url() );
-    Q_ASSERT(project);
+    if ( !project ) {
+        kDebug() << "no project found for" << job->url();
+        return;
+    }
 
     QList<KDevelop::ProjectFolderItem*> fitems = project->foldersForUrl( curUrl );
     if ( fitems.isEmpty() ) {
