@@ -72,6 +72,11 @@ class KDevProjectManagerViewFactory: public KDevelop::IToolViewFactory
         {
             return "org.kdevelop.ProjectsView";
         }
+        
+        virtual bool viewsWantProgressIndicator() const 
+        {
+            return true;
+        }
 
     private:
         ProjectManagerViewPlugin *mplugin;
@@ -94,10 +99,15 @@ public:
 };
 
 ProjectManagerViewPlugin::ProjectManagerViewPlugin( QObject *parent, const QVariantList& )
-        : IPlugin( ProjectManagerFactory::componentData(), parent ), d(new ProjectManagerViewPluginPrivate)
+        : IPlugin( ProjectManagerFactory::componentData(), parent ), d(new ProjectManagerViewPluginPrivate),
+        projectsBeingOpened( 0 )
 {
     d->buildSet = new ProjectBuildSetModel( this );
 
+    connect( core()->projectController(), SIGNAL( projectAboutToBeOpened( KDevelop::IProject* ) ),
+             SLOT( projectToBeOpened() ) );
+    connect( core()->projectController(), SIGNAL( projectOpened( KDevelop::IProject* ) ),
+             SLOT( projectOpened() ) );
     connect( core()->projectController(), SIGNAL( projectOpened( KDevelop::IProject* ) ),
              d->buildSet, SLOT( loadFromProject( KDevelop::IProject* ) ) );
 
@@ -146,6 +156,21 @@ ProjectManagerViewPlugin::ProjectManagerViewPlugin( QObject *parent, const QVari
              SLOT(updateFromBuildSetChange()));
 }
 
+void ProjectManagerViewPlugin::projectToBeOpened()
+{
+    projectsBeingOpened++;
+}
+
+void ProjectManagerViewPlugin::projectOpened()
+{
+    projectsBeingOpened--;
+}
+
+int ProjectManagerViewPlugin::numProjectsBeingOpened()
+{
+    return projectsBeingOpened;
+}
+
 void ProjectManagerViewPlugin::updateFromBuildSetChange()
 {
     updateActionState( core()->selectionController()->currentSelection() );
@@ -191,7 +216,7 @@ ContextMenuExtension ProjectManagerViewPlugin::contextMenuExtension( KDevelop::C
     ContextMenuExtension menuExt;
     bool closeProjectsAdded = false;
     bool buildItemsAdded = false;
-    bool hasTargets = false;
+    //bool hasTargets = false;
     bool folderItemsAdded = false;
     bool fileItemsAdded = false;
     bool folderWithParentAdded = false;
