@@ -58,6 +58,8 @@ public:
     KDevelop::ContextMenuExtension contextMenuExtension( KDevelop::Context* );
 
     ProjectBuildSetModel* buildSet();
+
+    int numProjectsBeingOpened();
 public Q_SLOTS:
     void buildProjectItems();
     void installProjectItems();
@@ -82,11 +84,14 @@ protected Q_SLOTS:
     void removeFileFromContextMenu();
     void updateActionState( KDevelop::Context* ctx );
     void updateFromBuildSetChange();
+    void projectToBeOpened();
+    void projectOpened();
 private:
     static KDevelop::IProjectBuilder* getProjectBuilder( KDevelop::ProjectBaseItem* item );
     QList<KDevelop::ProjectBaseItem*> recurseAndFetchCheckedItems( KDevelop::ProjectBaseItem* item );
     void runBuilderJob( KDevelop::BuilderJob::BuildType );
     class ProjectManagerViewPluginPrivate* const d;
+    int projectsBeingOpened;
 
 };
 
