@@ -78,6 +78,47 @@ using namespace KDevelop;
 
 const bool noHtmlDestriptionInOutline = true;
 
+class QuickOpenWidgetCreator {
+  public:
+    virtual ~QuickOpenWidgetCreator() {
+    }
+    virtual QuickOpenWidget* createWidget() = 0;
+    virtual QString objectNameForLine() = 0;
+    virtual void widgetShown() {
+    }
+};
+
+class StandardQuickOpenWidgetCreator : public QuickOpenWidgetCreator {
+  public:
+    StandardQuickOpenWidgetCreator(QStringList items, QStringList scopes) : m_items(items), m_scopes(scopes) {
+    }
+
+    virtual QString objectNameForLine() {
+      return "Quickopen";
+    }
+
+    void setItems(const QStringList& scopes, const QStringList& items)
+    {
+      m_scopes = scopes;
+      m_items = items;
+    }
+
+    virtual QuickOpenWidget* createWidget() {
+      QStringList useItems = m_items;
+      if(useItems.isEmpty())
+        useItems = QuickOpenPlugin::self()->lastUsedItems;
+      
+      QStringList useScopes = m_scopes;
+      if(useScopes.isEmpty())
+        useScopes = QuickOpenPlugin::self()->lastUsedScopes;
+      
+      return new QuickOpenWidget( i18n("Quick Open"), QuickOpenPlugin::self()->m_model, QuickOpenPlugin::self()->lastUsedItems, useScopes, false, true );
+    }
+    
+    QStringList m_items;
+    QStringList m_scopes;
+};
+
 class QuickOpenDelegate : public ExpandingDelegate {
 public:
   QuickOpenDelegate(ExpandingWidgetModel* model, QObject* parent = 0L) : ExpandingDelegate(model, parent) {
@@ -91,6 +132,35 @@ public:
 
 };
 
+class OutlineFilter : public DUChainUtils::DUChainItemFilter {
+public:
+  enum OutlineMode { Functions, FunctionsAndClasses };
+  OutlineFilter(QList<DUChainItem>& _items, OutlineMode _mode = FunctionsAndClasses) : items(_items), mode(_mode) {
+  }
+  virtual bool accept(Declaration* decl) {
+    if(decl->range().isEmpty())
+      return false;
+    bool collectable = mode == Functions ? decl->isFunctionDeclaration() : (decl->isFunctionDeclaration() || (decl->internalContext() && decl->internalContext()->type() == DUContext::Class));
+    if (collectable) {
+      DUChainItem item;
+      item.m_item = IndexedDeclaration(decl);
+      item.m_text = decl->toString();
+      items << item;
+
+      return true;
+    } else
+      return false;
+  }
+  virtual bool accept(DUContext* ctx) {
+    if(ctx->type() == DUContext::Class || ctx->type() == DUContext::Namespace || ctx->type() == DUContext::Global || ctx->type() == DUContext::Other || ctx->type() == DUContext::Helper )
+      return true;
+    else
+      return false;
+  }
+  QList<DUChainItem>& items;
+  OutlineMode mode;
+};
+
 K_PLUGIN_FACTORY(KDevQuickOpenFactory, registerPlugin<QuickOpenPlugin>(); )
 K_EXPORT_PLUGIN(KDevQuickOpenFactory(KAboutData("kdevquickopen","kdevquickopen", ki18n("Quick Open"), "0.1", ki18n("Quickly open resources such as files, classes and methods."), KAboutData::License_GPL)))
 
@@ -166,6 +236,8 @@ QString cursorItemText() {
 
   TopDUContext* context = DUChainUtils::standardContextForUrl( doc->url() );
 
+// Hallo hier steht ein mist
+
   if( !context ) {
     kDebug() << "Got no standard context";
     return QString();
@@ -179,12 +251,14 @@ QString cursorItemText() {
   if(!decl->qualifiedIdentifier().isEmpty())
     return decl->qualifiedIdentifier().last().identifier().str();
 
+// Und hier ein anderer
+
   return QString();
 }
 
-QWidget* QuickOpenPlugin::createQuickOpenLineWidget()
+QuickOpenLineEdit* QuickOpenPlugin::createQuickOpenLineWidget()
 {
-  return new QuickOpenLineEdit;
+  return new QuickOpenLineEdit(new StandardQuickOpenWidgetCreator(QStringList(), QStringList()));
 }
 
 void QuickOpenWidget::showStandardButtons(bool show)
@@ -200,6 +274,7 @@ void QuickOpenWidget::showStandardButtons(bool show)
 
 QuickOpenWidget::QuickOpenWidget( QString title, QuickOpenModel* model, const QStringList& initialItems, const QStringList& initialScopes, bool listOnly, bool noSearchField ) : m_model(model), m_expandedTemporary(false) {
 
+  Q_UNUSED( title );
   o.setupUi( this );
   o.list->header()->hide();
   o.list->setRootIsDecorated( false );
@@ -277,7 +352,7 @@ QuickOpenWidget::QuickOpenWidget( QString title, QuickOpenModel* model, const QS
 }
 
 
-void QuickOpenWidget::setAlternativeSearchField(QLineEdit* alterantiveSearchField)
+void QuickOpenWidget::setAlternativeSearchField(KLineEdit* alterantiveSearchField)
 {
     o.searchLine = alterantiveSearchField;
     o.searchLine->installEventFilter( this );
@@ -568,9 +643,9 @@ bool QuickOpenWidget::eventFilter ( QObject * watched, QEvent * event )
 }
 
 
-QuickOpenLineEdit* QuickOpenPlugin::quickOpenLine()
+QuickOpenLineEdit* QuickOpenPlugin::quickOpenLine(QString name)
 {
-  QList< QuickOpenLineEdit* > lines = ICore::self()->uiController()->activeMainWindow()->findChildren<QuickOpenLineEdit*>("Quickopen");
+  QList< QuickOpenLineEdit* > lines = ICore::self()->uiController()->activeMainWindow()->findChildren<QuickOpenLineEdit*>(name);
   foreach(QuickOpenLineEdit* line, lines) {
     if(line->isVisible()) {
       return line;
@@ -587,61 +662,80 @@ QuickOpenPlugin* QuickOpenPlugin::self()
   return staticQuickOpenPlugin;
 }
 
+void QuickOpenPlugin::createActionsForMainWindow(Sublime::MainWindow* /*window*/, QString& xmlFile, KActionCollection& actions)
+{
+    xmlFile ="kdevquickopen.rc";
+
+    KAction* quickOpen = actions.addAction("quick_open");
+    quickOpen->setText( i18n("&Quick Open") );
+    quickOpen->setIcon( KIcon("quickopen") );
+    quickOpen->setShortcut( Qt::CTRL | Qt::ALT | Qt::Key_Q );
+    connect(quickOpen, SIGNAL(triggered(bool)), this, SLOT(quickOpen()));
+
+    KAction* quickOpenFile = actions.addAction("quick_open_file");
+    quickOpenFile->setText( i18n("Quick Open &File") );
+    quickOpenFile->setIcon( KIcon("quickopen-file") );
+    quickOpenFile->setShortcut( Qt::CTRL | Qt::ALT | Qt::Key_O );
+    connect(quickOpenFile, SIGNAL(triggered(bool)), this, SLOT(quickOpenFile()));
+
+    KAction* quickOpenClass = actions.addAction("quick_open_class");
+    quickOpenClass->setText( i18n("Quick Open &Class") );
+    quickOpenClass->setIcon( KIcon("quickopen-class") );
+    quickOpenClass->setShortcut( Qt::CTRL | Qt::ALT | Qt::Key_C );
+    connect(quickOpenClass, SIGNAL(triggered(bool)), this, SLOT(quickOpenClass()));
+
+    KAction* quickOpenFunction = actions.addAction("quick_open_function");
+    quickOpenFunction->setText( i18n("Quick Open &Function") );
+    quickOpenFunction->setIcon( KIcon("quickopen-function") );
+    quickOpenFunction->setShortcut( Qt::CTRL | Qt::ALT | Qt::Key_M );
+    connect(quickOpenFunction, SIGNAL(triggered(bool)), this, SLOT(quickOpenFunction()));
+
+    KAction* quickOpenDeclaration = actions.addAction("quick_open_jump_declaration");
+    quickOpenDeclaration->setText( i18n("Jump to Declaration") );
+    quickOpenDeclaration->setIcon( KIcon("go-jump-declaration" ) );
+    quickOpenDeclaration->setShortcut( Qt::CTRL | Qt::Key_Period );
+    connect(quickOpenDeclaration, SIGNAL(triggered(bool)), this, SLOT(quickOpenDeclaration()));
+
+    KAction* quickOpenDefinition = actions.addAction("quick_open_jump_definition");
+    quickOpenDefinition->setText( i18n("Jump to Definition") );
+    quickOpenDefinition->setIcon( KIcon("go-jump-definition" ) );
+    quickOpenDefinition->setShortcut( Qt::CTRL | Qt::Key_Comma );
+    connect(quickOpenDefinition, SIGNAL(triggered(bool)), this, SLOT(quickOpenDefinition()));
+
+    KAction* quickOpenLine = actions.addAction("quick_open_line");
+    quickOpenLine->setText( i18n("Embedded Quick Open") );
+//     quickOpenLine->setShortcut( Qt::CTRL | Qt::ALT | Qt::Key_E );
+//     connect(quickOpenLine, SIGNAL(triggered(bool)), this, SLOT(quickOpenLine(bool)));
+    quickOpenLine->setDefaultWidget(createQuickOpenLineWidget());
+//     KAction* quickOpenNavigate = actions.addAction("quick_open_navigate");
+//     quickOpenNavigate->setText( i18n("Navigate Declaration") );
+//     quickOpenNavigate->setShortcut( Qt::ALT | Qt::Key_Space );
+//     connect(quickOpenNavigate, SIGNAL(triggered(bool)), this, SLOT(quickOpenNavigate()));
+
+    KAction* quickOpenNextFunction = actions.addAction("quick_open_next_function");
+    quickOpenNextFunction->setText( i18n("Next Function") );
+    quickOpenNextFunction->setShortcut( Qt::CTRL| Qt::ALT | Qt::Key_PageDown );
+    connect(quickOpenNextFunction, SIGNAL(triggered(bool)), this, SLOT(nextFunction()));
+
+    KAction* quickOpenPrevFunction = actions.addAction("quick_open_prev_function");
+    quickOpenPrevFunction->setText( i18n("Previous Function") );
+    quickOpenPrevFunction->setShortcut( Qt::CTRL| Qt::ALT | Qt::Key_PageUp );
+    connect(quickOpenPrevFunction, SIGNAL(triggered(bool)), this, SLOT(previousFunction()));
+
+    KAction* quickOpenNavigateFunctions = actions.addAction("quick_open_outline");
+    quickOpenNavigateFunctions->setText( i18n("Outline") );
+    quickOpenNavigateFunctions->setShortcut( Qt::CTRL| Qt::ALT | Qt::Key_N );
+    connect(quickOpenNavigateFunctions, SIGNAL(triggered(bool)), this, SLOT(quickOpenNavigateFunctions()));   
+}
+
 QuickOpenPlugin::QuickOpenPlugin(QObject *parent,
                                  const QVariantList&)
     : KDevelop::IPlugin(KDevQuickOpenFactory::componentData(), parent)
 {
     staticQuickOpenPlugin = this;
     KDEV_USE_EXTENSION_INTERFACE( KDevelop::IQuickOpen )
-    setXMLFile( "kdevquickopen.rc" );
     m_model = new QuickOpenModel( 0 );
 
-    KActionCollection* actions = actionCollection();
-
-    KAction* quickOpen = actions->addAction("quick_open");
-    quickOpen->setText( i18n("&Quick Open") );
-    quickOpen->setShortcut( Qt::CTRL | Qt::ALT | Qt::Key_Q );
-    connect(quickOpen, SIGNAL(triggered(bool)), this, SLOT(quickOpen()));
-
-    KAction* quickOpenFile = actions->addAction("quick_open_file");
-    quickOpenFile->setText( i18n("Quick Open &File") );
-    quickOpenFile->setShortcut( Qt::CTRL | Qt::ALT | Qt::Key_O );
-    connect(quickOpenFile, SIGNAL(triggered(bool)), this, SLOT(quickOpenFile()));
-
-    KAction* quickOpenClass = actions->addAction("quick_open_class");
-    quickOpenClass->setText( i18n("Quick Open &Class") );
-    quickOpenClass->setShortcut( Qt::CTRL | Qt::ALT | Qt::Key_C );
-    connect(quickOpenClass, SIGNAL(triggered(bool)), this, SLOT(quickOpenClass()));
-
-    KAction* quickOpenFunction = actions->addAction("quick_open_function");
-    quickOpenFunction->setText( i18n("Quick Open &Function") );
-    quickOpenFunction->setShortcut( Qt::CTRL | Qt::ALT | Qt::Key_M );
-    connect(quickOpenFunction, SIGNAL(triggered(bool)), this, SLOT(quickOpenFunction()));
-
-    KAction* quickOpenDeclaration = actions->addAction("quick_open_jump_declaration");
-    quickOpenDeclaration->setText( i18n("Jump to Declaration") );
-    quickOpenDeclaration->setShortcut( Qt::CTRL | Qt::Key_Period );
-    connect(quickOpenDeclaration, SIGNAL(triggered(bool)), this, SLOT(quickOpenDeclaration()));
-
-    KAction* quickOpenDefinition = actions->addAction("quick_open_jump_definition");
-    quickOpenDefinition->setText( i18n("Jump to Definition") );
-    quickOpenDefinition->setShortcut( Qt::CTRL | Qt::Key_Comma );
-    connect(quickOpenDefinition, SIGNAL(triggered(bool)), this, SLOT(quickOpenDefinition()));
-
-    KAction* quickOpenLine = actions->addAction("quick_open_line");
-    quickOpenLine->setText( i18n("Embedded Quick Open") );
-//     quickOpenLine->setShortcut( Qt::CTRL | Qt::ALT | Qt::Key_E );
-//     connect(quickOpenLine, SIGNAL(triggered(bool)), this, SLOT(quickOpenLine(bool)));
-    quickOpenLine->setDefaultWidget(createQuickOpenLineWidget());
-//     KAction* quickOpenNavigate = actions->addAction("quick_open_navigate");
-//     quickOpenNavigate->setText( i18n("Navigate Declaration") );
-//     quickOpenNavigate->setShortcut( Qt::ALT | Qt::Key_Space );
-//     connect(quickOpenNavigate, SIGNAL(triggered(bool)), this, SLOT(quickOpenNavigate()));
-
-    KAction* quickOpenNavigateFunctions = actions->addAction("quick_open_outline");
-    quickOpenNavigateFunctions->setText( i18n("Outline") );
-    quickOpenNavigateFunctions->setShortcut( Qt::CTRL| Qt::ALT | Qt::Key_N );
-    connect(quickOpenNavigateFunctions, SIGNAL(triggered(bool)), this, SLOT(quickOpenNavigateFunctions()));
     KConfigGroup quickopengrp = KGlobal::config()->group("QuickOpen");
     lastUsedScopes = quickopengrp.readEntry("SelectedScopes", QStringList() << i18n("Project") << i18n("Includes") << i18n("Includers") << i18n("Currently Open") );
     lastUsedItems = quickopengrp.readEntry("SelectedItems", QStringList() );
@@ -667,7 +761,6 @@ QuickOpenPlugin::QuickOpenPlugin(QObject *parent,
       items << ProjectItemDataProvider::supportedItemTypes();
       m_model->registerProvider( scopes, items, m_projectItemData );
     }
-
 }
 
 QuickOpenPlugin::~QuickOpenPlugin()
@@ -960,11 +1053,18 @@ bool QuickOpenPlugin::freeModel()
   return true;
 }
 
-void QuickOpenPlugin::quickOpenNavigateFunctions()
+void QuickOpenPlugin::nextFunction()
 {
-  if(!freeModel())
-    return;
+  jumpToNearestFunction(NextFunction);
+}
 
+void QuickOpenPlugin::previousFunction()
+{
+  jumpToNearestFunction(PreviousFunction);
+}
+
+void QuickOpenPlugin::jumpToNearestFunction(QuickOpenPlugin::FunctionJumpDirection direction)
+{
   IDocument* doc = ICore::self()->documentController()->activeDocument();
   if(!doc) {
     kDebug() << "No active document";
@@ -980,80 +1080,181 @@ void QuickOpenPlugin::quickOpenNavigateFunctions()
     return;
   }
 
-  QuickOpenModel* model = new QuickOpenModel(0);
-
   QList<DUChainItem> items;
-
-  class OutlineFilter : public DUChainUtils::DUChainItemFilter {
-  public:
-    OutlineFilter(QList<DUChainItem>& _items) : items(_items) {
-    }
-    virtual bool accept(Declaration* decl) {
-      if(decl->range().isEmpty())
-        return false;
-      if(decl->isFunctionDeclaration() || (decl->internalContext() && decl->internalContext()->type() == DUContext::Class)) {
-
-        DUChainItem item;
-        item.m_item = IndexedDeclaration(decl);
-        item.m_text = decl->toString();
-        items << item;
-
-        return true;
-      } else
-        return false;
-    }
-    virtual bool accept(DUContext* ctx) {
-      if(ctx->type() == DUContext::Class || ctx->type() == DUContext::Namespace || ctx->type() == DUContext::Global || ctx->type() == DUContext::Other || ctx->type() == DUContext::Helper )
-        return true;
-      else
-        return false;
-    }
-    QList<DUChainItem>& items;
-  } filter(items);
-
+  OutlineFilter filter(items, OutlineFilter::Functions);
   DUChainUtils::collectItems( context, filter );
 
-  if(noHtmlDestriptionInOutline) {
-    for(int a = 0; a < items.size(); ++a)
-      items[a].m_noHtmlDestription = true;
-  }
+  SimpleCursor cursor = SimpleCursor(doc->cursorPosition());
+  if (!cursor.isValid())
+    return;
 
-  Declaration* cursorDecl = cursorContextDeclaration();
+  Declaration *nearestDeclBefore = 0;
+  int distanceBefore = INT_MIN;
+  Declaration *nearestDeclAfter = 0;
+  int distanceAfter = INT_MAX;
 
-  model->registerProvider( QStringList(), QStringList(), new DeclarationListDataProvider(this, items, true) );
+  for (int i = 0; i < items.count(); ++i) {
+    Declaration *decl = items[i].m_item.data();
 
-  QuickOpenWidgetDialog* dialog = new QuickOpenWidgetDialog( i18n("Outline"), model, QStringList(), QStringList(), true );
-  m_currentWidgetHandler = dialog;
-  
-  model->setParent(dialog->widget());
-  
-  if(quickOpenLine()) {
-    quickOpenLine()->showWithWidget(dialog->widget());
-    dialog->deleteLater();
-  }else
-    dialog->run();
-
-  //Select the declaration that contains the cursor
-  if(cursorDecl) {
-    int num = 0;
-    foreach(const DUChainItem& item, items) {
-      if(item.m_item.data() == cursorDecl) {
-        dialog->widget()->o.list->setCurrentIndex( model->index(num,0,QModelIndex()) );
-        dialog->widget()->o.list->scrollTo( model->index(num,0,QModelIndex()), QAbstractItemView::PositionAtCenter );
-      }
-      ++num;
+    int distance = decl->range().start.line - cursor.line;
+    if (distance < 0 && distance >= distanceBefore) {
+      distanceBefore = distance;
+      nearestDeclBefore = decl;
+    } else if (distance > 0 && distance <= distanceAfter) {
+      distanceAfter = distance;
+      nearestDeclAfter = decl;
     }
   }
+
+  SimpleCursor c = SimpleCursor::invalid();
+  if (direction == QuickOpenPlugin::NextFunction && nearestDeclAfter)
+    c = nearestDeclAfter->range().start;
+  else if (direction == QuickOpenPlugin::PreviousFunction && nearestDeclBefore)
+    c = nearestDeclBefore->range().start;
+
+  lock.unlock();
+  if (c.isValid())
+    core()->documentController()->openDocument(doc->url(), c.textCursor());
+  else
+    kDebug() << "No declaration to jump to";
 }
-QuickOpenLineEdit::QuickOpenLineEdit() : m_widget(0), m_forceUpdate(false) {
+
+
+struct CreateOutlineDialog {
+  CreateOutlineDialog() : dialog(0), cursorDecl(0), model(0) {
+  }
+  
+  void start() {
+    if(!QuickOpenPlugin::self()->freeModel())
+        return;
+
+      IDocument* doc = ICore::self()->documentController()->activeDocument();
+      if(!doc) {
+        kDebug() << "No active document";
+        return;
+      }
+
+      KDevelop::DUChainReadLocker lock( DUChain::lock() );
+
+      TopDUContext* context = DUChainUtils::standardContextForUrl( doc->url() );
+
+      if( !context ) {
+        kDebug() << "Got no standard context";
+        return;
+      }
+
+      model = new QuickOpenModel(0);
+
+      OutlineFilter filter(items);
+
+      DUChainUtils::collectItems( context, filter );
+
+      if(noHtmlDestriptionInOutline) {
+        for(int a = 0; a < items.size(); ++a)
+          items[a].m_noHtmlDestription = true;
+      }
+
+      cursorDecl = cursorContextDeclaration();
+
+      model->registerProvider( QStringList(), QStringList(), new DeclarationListDataProvider(QuickOpenPlugin::self(), items, true) );
+
+      dialog = new QuickOpenWidgetDialog( i18n("Outline"), model, QStringList(), QStringList(), true );
+      
+      model->setParent(dialog->widget());    
+  }
+  void finish() {
+    //Select the declaration that contains the cursor
+    if(cursorDecl && dialog) {
+      int num = 0;
+      foreach(const DUChainItem& item, items) {
+        if(item.m_item.data() == cursorDecl) {
+          dialog->widget()->o.list->setCurrentIndex( model->index(num,0,QModelIndex()) );
+          dialog->widget()->o.list->scrollTo( model->index(num,0,QModelIndex()), QAbstractItemView::PositionAtCenter );
+        }
+        ++num;
+      }
+    }
+  }
+    QPointer<QuickOpenWidgetDialog> dialog;
+    Declaration* cursorDecl;
+    QList<DUChainItem> items;
+    QuickOpenModel* model;
+};
+
+class OutlineQuickopenWidgetCreator : public QuickOpenWidgetCreator {
+  public:
+    OutlineQuickopenWidgetCreator(QStringList /*scopes*/, QStringList /*items*/) : m_creator(0) {
+    }
+    
+    ~OutlineQuickopenWidgetCreator() {
+      delete m_creator;
+    }
+    
+    virtual QuickOpenWidget* createWidget() {
+      delete m_creator;
+      m_creator = new CreateOutlineDialog;
+      m_creator->start();
+      
+      if(!m_creator->dialog)
+        return 0;
+      
+      m_creator->dialog->deleteLater();
+      return m_creator->dialog->widget();
+    }
+    
+    virtual void widgetShown() {
+      if(m_creator) {
+        m_creator->finish();
+        delete m_creator;
+        m_creator = 0;
+      }
+    }
+    
+    virtual QString objectNameForLine() {
+      return "Outline";
+    }
+    
+    CreateOutlineDialog* m_creator;
+};
+
+void QuickOpenPlugin::quickOpenNavigateFunctions()
+{
+  CreateOutlineDialog create;
+  create.start();
+  
+  if(!create.dialog)
+    return;
+  
+  m_currentWidgetHandler = create.dialog;
+  
+  QuickOpenLineEdit* line = quickOpenLine("Outline");
+  if(!line)
+    line  = quickOpenLine();
+  
+  if(line) {
+    line->showWithWidget(create.dialog->widget());
+    create.dialog->deleteLater();
+  }else
+    create.dialog->run();
+  
+  create.finish();
+}
+
+QuickOpenLineEdit::QuickOpenLineEdit(QuickOpenWidgetCreator* creator) : m_widget(0), m_forceUpdate(false), m_widgetCreator(creator) {
     setMinimumWidth(200);
     setMaximumWidth(400);
+
     deactivate();
-    setObjectName("Quickopen");
+    setDefaultText("Quick Open...");
+    setObjectName(m_widgetCreator->objectNameForLine());
+    setFocusPolicy(Qt::ClickFocus);
 }
+
 QuickOpenLineEdit::~QuickOpenLineEdit() {
     delete m_widget;
+    delete m_widgetCreator;
 }
+
 void QuickOpenLineEdit::keyPressEvent(QKeyEvent* ev) {
     if (ev->key() == Qt::Key_Escape) {
       kDebug() << "escape";
@@ -1062,6 +1263,7 @@ void QuickOpenLineEdit::keyPressEvent(QKeyEvent* ev) {
     }
     QLineEdit::keyPressEvent(ev);
 }
+
 bool QuickOpenLineEdit::insideThis(QObject* object) {
     while (object)
     {
@@ -1096,18 +1298,26 @@ void QuickOpenLineEdit::focusInEvent(QFocusEvent* ev) {
     if (m_widget && !m_forceUpdate)
         return;
 
-    if (!m_forceUpdate && !QuickOpenPlugin::self()->freeModel())
+    if (!m_forceUpdate && !QuickOpenPlugin::self()->freeModel()) {
+        deactivate();
         return;
+    }
     
-    activate();
     m_forceUpdate = false;
     
-    if(!m_widget) {
-      m_widget = new QuickOpenWidget( i18n("Quick Open"), QuickOpenPlugin::self()->m_model, QuickOpenPlugin::self()->lastUsedItems, QuickOpenPlugin::self()->lastUsedScopes, false, true );
-    }else{
-      m_widget->showStandardButtons(false);
-      m_widget->showSearchField(false);
+    if(!m_widget)
+    {
+      m_widget = m_widgetCreator->createWidget();
+      if(!m_widget) {
+        deactivate();
+        return;
+      }
     }
+
+    activate();
+
+    m_widget->showStandardButtons(false);
+    m_widget->showSearchField(false);
     
     m_widget->setParent(0, Qt::ToolTip);
     m_widget->setFocusPolicy(Qt::NoFocus);
@@ -1134,6 +1344,8 @@ void QuickOpenLineEdit::focusInEvent(QFocusEvent* ev) {
     m_widget->setFrameRect(frameRect);
 
     m_widget->show();
+    
+    m_widgetCreator->widgetShown();
 }
 
 void QuickOpenLineEdit::hideEvent(QHideEvent* ev)
@@ -1185,12 +1397,14 @@ void QuickOpenLineEdit::activate() {
 void QuickOpenLineEdit::deactivate() {
     kDebug() << "deactivating";
     
-    setText(i18n("Quick Open"));
-    setStyleSheet("color: grey"); ///@todo Better color picking
-    if (m_widget) {
+    clear();
+
+    if(m_widget || hasFocus())
+      QMetaObject::invokeMethod(this, "checkFocus", Qt::QueuedConnection);
+    
+   if (m_widget)
         m_widget->deleteLater();
-        QMetaObject::invokeMethod(this, "checkFocus", Qt::QueuedConnection);
-    }
+    
     m_widget = 0;
     qApp->removeEventFilter(this);
     
@@ -1207,9 +1421,20 @@ void QuickOpenLineEdit::checkFocus()
     }else{
        if (ICore::self()->documentController()->activeDocument())
            ICore::self()->documentController()->activateDocument(ICore::self()->documentController()->activeDocument());
+       
+       //Make sure the focus is somewehre else, even if there is no active document
+       setEnabled(false);
+       setEnabled(true);
     }
 }
 
+IQuickOpenLine* QuickOpenPlugin::createQuickOpenLine(const QStringList& scopes, const QStringList& type, IQuickOpen::QuickOpenType kind)
+{
+  if(kind == Outline)
+    return new QuickOpenLineEdit(new OutlineQuickopenWidgetCreator(scopes, type));
+  else
+    return new QuickOpenLineEdit(new StandardQuickOpenWidgetCreator(scopes, type));
+}
 
 #include "quickopenplugin.moc"
 
