@@ -47,6 +47,7 @@
 #include "runcontroller.h"
 #include "documentationcontroller.h"
 #include "sourceformattercontroller.h"
+#include "progressmanager.h"
 #include "selectioncontroller.h"
 #include "debugcontroller.h"
 #include "kdevplatformversion.h"
@@ -135,6 +136,11 @@ void CorePrivate::initialize(Core::Setup mode)
         sourceFormatterController = new SourceFormatterController(m_core);
     }
 
+    if ( !progressController) 
+    {
+        progressController = new ProgressManager();
+    }
+
     if( !selectionController )
     {
         selectionController = new SelectionController(m_core);
@@ -306,7 +312,7 @@ ProjectController *Core::projectControllerInternal()
     return d->projectController;
 }
 
-KParts::PartManager *Core::partController()
+IPartController *Core::partController()
 {
     return d->partController;
 }
@@ -351,6 +357,11 @@ ISourceFormatterController* Core::sourceFormatterController()
     return d->sourceFormatterController;
 }
 
+ProgressManager *Core::progressController()
+{
+    return d->progressController;
+}
+
 ISelectionController* Core::selectionController()
 {
     return d->selectionController;
