@@ -38,15 +38,18 @@
 #include "../interfaces/idocument.h"
 #include "../interfaces/icore.h"
 #include "../interfaces/idocumentcontroller.h"
+#include "../interfaces/ipartcontroller.h"
 #include "../interfaces/iuicontroller.h"
 #include "../interfaces/contextmenuextension.h"
 #include "../interfaces/context.h"
 #include "../language/interfaces/editorcontext.h"
 #include "../sublime/view.h"
-#include "debugger/framestackwidget.h"
 #include "core.h"
 #include "../debugger/breakpoint/breakpointmodel.h"
 #include "../debugger/breakpoint/breakpointwidget.h"
+#include "../debugger/variable/variablewidget.h"
+#include "../debugger/framestack/framestackmodel.h"
+#include "../debugger/framestack/framestackwidget.h"
 
 
 namespace KDevelop {
@@ -94,7 +97,10 @@ private:
 };
 
 DebugController::DebugController(QObject *parent)
-    : IDebugController(parent), KXMLGUIClient(), m_breakpointModel(new BreakpointModel(this))
+    : IDebugController(parent), KXMLGUIClient(),
+      m_breakpointModel(new BreakpointModel(this)),
+      m_variableCollection(new VariableCollection(this)),
+      m_frameStackModel(new FrameStackModel(this))
 {
     setComponentData(KComponentData("kdevdebugger"));
     setXMLFile("kdevdebuggershellui.rc");
@@ -114,6 +120,12 @@ DebugController::DebugController(QObject *parent)
             this, "org.kdevelop.debugger.BreakpointsView",
             Qt::BottomDockWidgetArea));
 
+    ICore::self()->uiController()->addToolView(
+        i18n("Variables"),
+        new DebuggerToolFactory<VariableWidget>(
+            this, "org.kdevelop.debugger.VariablesView",
+            Qt::LeftDockWidgetArea));
+
     foreach(KParts::Part* p, KDevelop::ICore::self()->partController()->parts())
         partAdded(p);
     connect(KDevelop::ICore::self()->partController(),
@@ -133,6 +145,18 @@ BreakpointModel* DebugController::breakpointModel()
     return m_breakpointModel;
 }
 
+VariableCollection* DebugController::variableCollection()
+{
+    return m_variableCollection;
+}
+
+FrameStackModel* DebugController::frameStackModel()
+{
+    return m_frameStackModel;
+}
+
+
+
 void DebugController::partAdded(KParts::Part* part)
 {
     if (KTextEditor::Document* doc = dynamic_cast<KTextEditor::Document*>(part)) {
@@ -271,6 +295,10 @@ void DebugController::addSession(IDebugSession* session)
     updateDebuggerState(session->state(), session);
     
     emit sessionAdded(session);
+    
+    if((Core::self()->setupFlags() & Core::NoUi)) return;
+
+    ICore::self()->uiController()->switchToArea("debug", IUiController::ThisWindow);
 }
 
 void DebugController::clearExecutionPoint()
@@ -293,6 +321,8 @@ void DebugController::clearExecutionPoint()
 
 void DebugController::showStepInSource(const KUrl &url, int lineNum)
 {
+    if((Core::self()->setupFlags() & Core::NoUi)) return;
+
     clearExecutionPoint();
     kDebug() << url << lineNum;
     KDevelop::IDocument* document = KDevelop::ICore::self()
@@ -331,6 +361,8 @@ void DebugController::debuggerStateChanged(KDevelop::IDebugSession::DebuggerStat
 
 void DebugController::updateDebuggerState(IDebugSession::DebuggerState state, IDebugSession *session)
 {
+    if((Core::self()->setupFlags() & Core::NoUi)) return;
+
     kDebug() << state;
     switch (state) {
         case IDebugSession::StoppedState:
