@@ -51,6 +51,8 @@ public:
     virtual ContextMenuExtension contextMenuExtension( Context* context );
 
     virtual BreakpointModel* breakpointModel();
+    virtual VariableCollection* variableCollection();
+    virtual FrameStackModel *frameStackModel();
 
 private Q_SLOTS:
     void restartDebugger();
@@ -96,6 +98,8 @@ private:
 
     QPointer<IDebugSession> m_currentSession;
     BreakpointModel *m_breakpointModel;
+    VariableCollection *m_variableCollection;
+    FrameStackModel *m_frameStackModel;
 };
 
 }
