@@ -51,7 +51,12 @@ Boston, MA 02110-1301, USA.
 #include "uicontroller.h"
 #include "partcontroller.h"
 #include "savedialog.h"
+#include <kmessagebox.h>
 
+#include <config-kdevplatform.h>
+#if HAVE_KOMPARE
+    #include "patchdocument.h"
+#endif
 
 namespace KDevelop
 {
@@ -129,6 +134,240 @@ struct DocumentControllerPrivate {
         }
     }
 
+    IDocument* openDocumentInternal( const KUrl & inputUrl, const QString& prefName = QString(),
+        const KTextEditor::Range& range = KTextEditor::Range::invalid(),
+        DocumentController::DocumentActivationParams activationParams = 0 )
+    {
+        IDocument* previousActiveDocument = controller->activeDocument();
+        KTextEditor::Cursor previousActivePosition;
+        if(previousActiveDocument && previousActiveDocument->textDocument() && previousActiveDocument->textDocument()->activeView())
+            previousActivePosition = previousActiveDocument->textDocument()->activeView()->cursorPosition();
+        
+
+        UiController *uiController = Core::self()->uiControllerInternal();
+        Sublime::Area *area = uiController->activeArea();
+
+        KUrl url = inputUrl;
+        QString encoding = "";
+
+        if ( url.isEmpty() && (!activationParams.testFlag(IDocumentController::DoNotCreateView)) )
+        {
+            KUrl dir;
+            if( controller->activeDocument() )
+            {
+                dir = KUrl( controller->activeDocument()->url().directory() );
+            }else
+            {
+                dir = KGlobal::config()->group("Open File").readEntry( "Last Open File Directory", Core::self()->projectController()->projectsBaseDirectory() );
+            }
+
+            KEncodingFileDialog::Result res = KEncodingFileDialog::getOpenUrlAndEncoding( "", dir.url(), i18n( "*|Text File\n" ),
+                                        Core::self()->uiControllerInternal()->defaultMainWindow(),
+                                        i18n( "Open File" ) );
+            if( !res.URLs.isEmpty() )
+                url = res.URLs.first();
+            encoding = res.encoding;
+        }
+        if ( url.isEmpty() )
+            //still no url
+            return 0;
+
+        KGlobal::config()->group("Open File").writeEntry( "Last Open File Directory", url.upUrl() );
+
+        // clean it and resolve possible symlink
+        url.cleanPath( KUrl::SimplifyDirSeparators );
+        if ( url.isLocalFile() )
+        {
+            QString path = QFileInfo( url.toLocalFile() ).canonicalFilePath();
+            if ( !path.isEmpty() )
+                url.setPath( path );
+        }
+        
+        //get a part document
+        IDocument* doc=0;
+        if (documents.contains(url))
+            doc=documents.value(url);
+        else
+        {
+            KMimeType::Ptr mimeType;
+
+            if (DocumentController::isEmptyDocumentUrl(url))
+            {
+                mimeType = KMimeType::mimeType("text/plain");
+            }
+            else
+            {
+                //make sure the URL exists
+                if ( !url.isValid() || !KIO::NetAccess::exists( url, KIO::NetAccess::DestinationSide, 0 ) )
+                {
+                    kDebug() << "cannot find URL:" << url.url();
+                    return 0;
+                }
+
+                mimeType = KMimeType::findByUrl( url );
+            }
+
+            // is the URL pointing to a directory?
+            if ( mimeType->is( "inode/directory" ) )
+            {
+                kDebug() << "cannot open directory:" << url.url();
+                return 0;
+            }
+
+            if( prefName.isEmpty() )
+            {
+                // Try to find a plugin that handles this mimetype
+                QString constraint = QString("'%1' in [X-KDevelop-SupportedMimeTypes]").arg(mimeType->name());
+                KPluginInfo::List plugins = IPluginController::queryPlugins( constraint );
+
+                if( !plugins.isEmpty() )
+                {
+                    KPluginInfo info = plugins.first();
+                    Core::self()->pluginController()->loadPlugin( info.pluginName() );
+                }
+            }
+            
+            if( factories.contains( mimeType->name() ) )
+            {
+                doc = factories[mimeType->name()]->create(url, Core::self());
+            }
+            
+            if(!doc) {
+                if( !prefName.isEmpty() ) 
+                {
+                    doc = new PartDocument(url, Core::self(), prefName);
+                } else  if ( Core::self()->partControllerInternal()->isTextType(mimeType)) 
+                {
+                    doc = new TextDocument(url, Core::self(), encoding);
+                } else if( Core::self()->partControllerInternal()->canCreatePart(url) ) 
+                {
+                    doc = new PartDocument(url, Core::self());
+                } else
+                {
+                    int openAsText = KMessageBox::questionYesNo(0, i18n("KDevelop could not find the editor for file '%1'.\nDo you want to open it as plain text?", url.fileName()), i18n("Could Not Find Editor"));
+                    if (openAsText == KMessageBox::Yes)
+                        doc = new TextDocument(url, Core::self(), encoding);
+                    else
+                        return 0;
+                }
+            }
+        }
+        
+        if(doc && openDocumentInternal(doc, range, activationParams))
+            return doc;
+        else
+            return 0;
+        
+    }
+    
+    bool openDocumentInternal(IDocument* doc,
+                                const KTextEditor::Range& range,
+                                DocumentController::DocumentActivationParams activationParams)
+    {
+        IDocument* previousActiveDocument = controller->activeDocument();
+        KTextEditor::Cursor previousActivePosition;
+        if(previousActiveDocument && previousActiveDocument->textDocument() && previousActiveDocument->textDocument()->activeView())
+            previousActivePosition = previousActiveDocument->textDocument()->activeView()->cursorPosition();
+        
+        KUrl url=doc->url();
+        UiController *uiController = Core::self()->uiControllerInternal();
+        Sublime::Area *area = uiController->activeArea();
+        
+        //We can't have the same url in many documents
+        //so we check it's already the same if it exists
+        //contains=>it's the same
+        Q_ASSERT(!documents.contains(url) || documents[url]==doc);
+
+        Sublime::Document *sdoc = dynamic_cast<Sublime::Document*>(doc);
+        if( !sdoc )
+        {
+            documents.remove(url);
+            delete doc;
+            return false;
+        }
+        //react on document deletion - we need to cleanup controller structures
+        QObject::connect(sdoc, SIGNAL(aboutToDelete(Sublime::Document*)), controller, SLOT(removeDocument(Sublime::Document*)));
+        //We check if it was already opened before
+        bool emitOpened = !documents.contains(url);
+        if(emitOpened)
+            documents[url]=doc;
+        
+        if (!activationParams.testFlag(IDocumentController::DoNotCreateView))
+        {
+            //find a view if there's one already opened in this area
+            Sublime::View *partView = 0;
+            foreach (Sublime::View *view, sdoc->views())
+            {
+                if (area->views().contains(view) && (activationParams.testFlag(DocumentController::DoNotForceCurrentView) || area->indexOf(view) == area->indexOf(uiController->activeSublimeWindow()->activeView())))
+                {
+                    partView = view;
+                    break;
+                }
+            }
+            bool addView = false, applyRange = true;
+            if (!partView)
+            {
+                //no view currently shown for this url
+                partView = sdoc->createView();
+                addView = true;
+            }
+            
+            KDevelop::TextView* textView = dynamic_cast<KDevelop::TextView*>(partView);
+            if(textView && textView->textView()) {
+                applyRange = false;
+                if (range.isEmpty())
+                    doc->setCursorPosition( range.start() );
+                else
+                    doc->setTextSelection( range );
+            }else if(textView) {
+                textView->setInitialRange(range);
+            }
+            
+            if(addView) {
+                //add view to the area
+                area->addView(partView, uiController->activeSublimeWindow()->activeView());
+            }
+            
+            if (!activationParams.testFlag(IDocumentController::DoNotActivate))
+            {
+                uiController->activeSublimeWindow()->activateView(partView);
+            }
+            fileOpenRecent->addUrl( url );
+
+            if( applyRange && range.isValid() )
+            {
+                if (range.isEmpty())
+                    doc->setCursorPosition( range.start() );
+                else
+                    doc->setTextSelection( range );
+            }
+        }
+
+        // Deferred signals, wait until it's all ready first
+        if( emitOpened ) {
+            emit controller->documentOpened( doc );
+        }
+
+        if (!activationParams.testFlag(IDocumentController::DoNotActivate) && doc != controller->activeDocument())
+            emit controller->documentActivated( doc );
+
+        saveAll->setEnabled(true);
+        revertAll->setEnabled(true);
+        close->setEnabled(true);
+        closeAll->setEnabled(true);
+        closeAllOthers->setEnabled(true);
+
+        if(doc) {
+            KTextEditor::Cursor activePosition;
+            if(doc->textDocument() && doc->textDocument()->activeView())
+                activePosition = doc->textDocument()->activeView()->cursorPosition();
+            
+            emit controller->documentJumpPerformed(doc, range.isValid() ? range.start() : activePosition, previousActiveDocument, previousActivePosition);
+        }
+        
+        return true;
+    }
+
     DocumentController* controller;
 
     QList<HistoryEntry> backHistory;
@@ -159,6 +398,10 @@ DocumentController::DocumentController( QObject *parent )
     connect(this, SIGNAL(documentUrlChanged(KDevelop::IDocument*)), this, SLOT(changeDocumentUrl(KDevelop::IDocument*)));
 
     if(!(Core::self()->setupFlags() & Core::NoUi)) setupActions();
+
+#if HAVE_KOMPARE    
+    registerDocumentForMimetype("text/x-patch", new PatchDocumentFactory);
+#endif
 }
 
 void KDevelop::DocumentController::initialize()
@@ -262,197 +505,32 @@ void DocumentController::slotOpenDocument(const KUrl &url)
 
 IDocument* DocumentController::openDocumentFromText( const QString& data )
 {
-    KTemporaryFile *temp = new KTemporaryFile();
-    temp->setSuffix("kdevtmp");
-    temp->open();
-    temp->write(data.toUtf8());
-    temp->flush();
-    d->tempFiles << temp;
-    return openDocument(temp->fileName());
+    IDocument* d = openDocument(nextEmptyDocumentUrl());
+    d->textDocument()->setText( data );
+    return d;
+}
+
+IDocument* DocumentController::openDocument( const KUrl& inputUrl, const QString& prefName )
+{
+    return d->openDocumentInternal( inputUrl, prefName );
 }
 
 IDocument* DocumentController::openDocument( const KUrl & inputUrl,
         const KTextEditor::Range& range,
         DocumentActivationParams activationParams)
 {
-    IDocument* previousActiveDocument = activeDocument();
-    KTextEditor::Cursor previousActivePosition;
-    if(previousActiveDocument && previousActiveDocument->textDocument() && previousActiveDocument->textDocument()->activeView())
-        previousActivePosition = previousActiveDocument->textDocument()->activeView()->cursorPosition();
-    
-
-    UiController *uiController = Core::self()->uiControllerInternal();
-    Sublime::Area *area = uiController->activeArea();
-
-    KUrl url = inputUrl;
-    QString encoding = "";
-
-    if ( url.isEmpty() && (!activationParams.testFlag(IDocumentController::DoNotCreateView)) )
-    {
-        KUrl dir;
-        if( activeDocument() )
-        {
-            dir = KUrl( activeDocument()->url().directory() );
-        }else
-        {
-            dir = KGlobal::config()->group("Open File").readEntry( "Last Open File Directory", Core::self()->projectController()->projectsBaseDirectory() );
-        }
-
-        KEncodingFileDialog::Result res = KEncodingFileDialog::getOpenUrlAndEncoding( "", dir.url(), i18n( "*.*|Text File\n" ),
-                                       Core::self()->uiControllerInternal()->defaultMainWindow(),
-                                       i18n( "Open File" ) );
-        if( !res.URLs.isEmpty() )
-            url = res.URLs.first();
-        encoding = res.encoding;
-    }
-    if ( url.isEmpty() )
-        //still no url
-        return 0;
-
-    KGlobal::config()->group("Open File").writeEntry( "Last Open File Directory", url.upUrl() );
-
-    bool emitOpened = false;
-
-    //get a part document
-    if (!d->documents.contains(url))
-    {
-        //make sure the URL exists
-        if ( !url.isValid() || !KIO::NetAccess::exists( url, KIO::NetAccess::DestinationSide, 0 ) )
-        {
-            kDebug() << "cannot find URL:" << url.url();
-            return 0;
-        }
-
-        // clean it and resolve possible symlink
-        url.cleanPath( KUrl::SimplifyDirSeparators );
-        if ( url.isLocalFile() )
-        {
-            QString path = QFileInfo( url.toLocalFile() ).canonicalFilePath();
-            if ( !path.isEmpty() )
-                url.setPath( path );
-        }
-
-        KMimeType::Ptr mimeType = KMimeType::findByUrl( url );
-
-        // is the URL pointing to a directory?
-        if ( mimeType->is( "inode/directory" ) )
-        {
-            kDebug() << "cannot open directory:" << url.url();
-            return 0;
-        }
-
-        // Try to find a plugin that handles this mimetype
-        QString constraint = QString("'%1' in [X-KDevelop-SupportedMimeTypes]").arg(mimeType->name());
-        KPluginInfo::List plugins = IPluginController::queryPlugins( constraint );
-
-        if( !plugins.isEmpty() )
-        {
-            KPluginInfo info = plugins.first();
-            kDebug() << "loading" << info.pluginName();
-            Core::self()->pluginController()->loadPlugin( info.pluginName() );
-            if( d->factories.contains( mimeType->name() ) )
-            {
-                IDocument* idoc = d->factories[mimeType->name()]->create(url, Core::self());
-                if( idoc )
-                {
-                     d->documents[url] = idoc;
-                }
-            }
-        }
-        if ( !d->documents.contains(url) && Core::self()->partControllerInternal()->isTextType(mimeType))
-            d->documents[url] = new TextDocument(url, Core::self(), encoding);
-        else if( !d->documents.contains(url) )
-            d->documents[url] = new PartDocument(url, Core::self());
-        emitOpened = d->documents.contains(url);
-    }
-    IDocument *doc = d->documents[url];
-
-    Sublime::Document *sdoc = dynamic_cast<Sublime::Document*>(doc);
-    if( !sdoc )
-    {
-        d->documents.remove(url);
-        delete doc;
-        return 0;
-    }
-    //react on document deletion - we need to cleanup controller structures
-    connect(sdoc, SIGNAL(aboutToDelete(Sublime::Document*)), this, SLOT(removeDocument(Sublime::Document*)));
-
-    if (!activationParams.testFlag(IDocumentController::DoNotCreateView))
-    {
-        //find a view if there's one already opened in this area
-        Sublime::View *partView = 0;
-        foreach (Sublime::View *view, sdoc->views())
-        {
-            if (area->views().contains(view) && (activationParams.testFlag(DoNotForceCurrentView) || area->indexOf(view) == area->indexOf(uiController->activeSublimeWindow()->activeView())))
-            {
-                partView = view;
-                break;
-            }
-        }
-        bool addView = false, applyRange = true;
-        if (!partView)
-        {
-            //no view currently shown for this url
-            partView = sdoc->createView();
-            addView = true;
-        }
-        
-        KDevelop::TextView* textView = dynamic_cast<KDevelop::TextView*>(partView);
-        if(textView && textView->textView()) {
-            applyRange = false;
-            if (range.isEmpty())
-                doc->setCursorPosition( range.start() );
-            else
-                doc->setTextSelection( range );
-        }else if(textView) {
-            textView->setInitialRange(range);
-        }
-        
-        if(addView) {
-            //add view to the area
-            area->addView(partView, uiController->activeSublimeWindow()->activeView());
-        }
-        
-        if (!activationParams.testFlag(IDocumentController::DoNotActivate))
-        {
-            uiController->activeSublimeWindow()->activateView(partView);
-        }
-        d->fileOpenRecent->addUrl( url );
-
-        if( applyRange && range.isValid() )
-        {
-            if (range.isEmpty())
-                doc->setCursorPosition( range.start() );
-            else
-                doc->setTextSelection( range );
-        }
-    }
-
-    // Deferred signals, wait until it's all ready first
-    if( emitOpened ) {
-        emit documentOpened( d->documents[url] );
-    }
-
-    if (!activationParams.testFlag(IDocumentController::DoNotActivate) && doc != activeDocument())
-        emit documentActivated( doc );
-
-    d->saveAll->setEnabled(true);
-    d->revertAll->setEnabled(true);
-    d->close->setEnabled(true);
-    d->closeAll->setEnabled(true);
-    d->closeAllOthers->setEnabled(true);
-
-    if(doc) {
-        KTextEditor::Cursor activePosition;
-        if(doc->textDocument() && doc->textDocument()->activeView())
-            activePosition = doc->textDocument()->activeView()->cursorPosition();
-        
-        emit documentJumpPerformed(doc, range.start(), previousActiveDocument, previousActivePosition);
-    }
-    
-    return doc;
+    return d->openDocumentInternal( inputUrl, "", range, activationParams );
 }
 
+
+bool DocumentController::openDocument(IDocument* doc,
+                                      const KTextEditor::Range& range,
+                                      DocumentActivationParams activationParams)
+{
+    return d->openDocumentInternal( doc, range, activationParams );
+}
+
+
 void DocumentController::fileClose()
 {
     IDocument *activeDoc = activeDocument();
@@ -496,8 +574,11 @@ void DocumentController::notifyDocumentClosed(IDocument* doc)
     emit documentClosed(doc);
 }
 
-IDocument * DocumentController::documentForUrl( const KUrl & url ) const
+IDocument * DocumentController::documentForUrl( const KUrl & dirtyUrl ) const
 {
+    //Fix urls that might not be absolute
+    KUrl url(dirtyUrl);
+    url.cleanPath();
     if ( d->documents.contains( url ) )
         return d->documents.value( url );
 
@@ -540,8 +621,13 @@ bool KDevelop::DocumentController::saveSomeDocuments(const QList< IDocument * >
 {
     if (mode & IDocument::Silent) {
         foreach (IDocument* doc, modifiedDocuments(list)) {
-            bool ret = doc->save(mode);
-            Q_ASSERT(ret);
+            if( !doc->save(mode) )
+            {
+                if( doc )
+                    qWarning() << "!! Could not save document:" << doc->url();
+                else
+                    qWarning() << "!! Could not save document as its NULL";
+            }
             // TODO if (!ret) showErrorDialog() ?
         }
 
@@ -687,6 +773,40 @@ QStringList DocumentController::documentTypes() const
     return QStringList() << "Text";
 }
 
+bool DocumentController::isEmptyDocumentUrl(const KUrl &url)
+{
+    QRegExp r(QString("^%1(\\s\\(\\d+\\))?$").arg(EMPTY_DOCUMENT_URL));
+    return r.indexIn(url.prettyUrl()) != -1;
+}
+
+KUrl DocumentController::nextEmptyDocumentUrl()
+{
+    int nextEmptyDocNumber = 0;
+    foreach (IDocument *doc, Core::self()->documentControllerInternal()->openDocuments())
+    {
+        if (DocumentController::isEmptyDocumentUrl(doc->url()))
+        {
+            QRegExp r(QString("^%1\\s\\((\\d+)\\)$").arg(EMPTY_DOCUMENT_URL));
+            if (r.indexIn(doc->url().prettyUrl()) != -1)
+                nextEmptyDocNumber = qMax(nextEmptyDocNumber, r.cap(1).toInt()+1);
+            else
+                nextEmptyDocNumber = qMax(nextEmptyDocNumber, 1);
+        }
+    }
+    
+    KUrl url;
+    if (nextEmptyDocNumber > 0)
+        url = KUrl(QString("%1 (%2)").arg(EMPTY_DOCUMENT_URL).arg(nextEmptyDocNumber));
+    else
+        url = KUrl(EMPTY_DOCUMENT_URL);
+    return url;
+}
+
+IDocumentFactory* DocumentController::factory(const QString& mime) const
+{
+    return d->factories.value(mime);
+}
+
 }
 
 #include "documentcontroller.moc"
