@@ -41,9 +41,9 @@
 #include "runcontroller.h"
 #include "launchconfiguration.h"
 
-#include "ui_launchconfigtypepage.h"
 #include <interfaces/ilauncher.h>
 #include <interfaces/ilaunchmode.h>
+#include <QLayout>
 
 namespace KDevelop
 {
@@ -53,6 +53,7 @@ LaunchConfigurationDialog::LaunchConfigurationDialog(QWidget* parent): KDialog(p
 {
     setButtons( KDialog::Ok | KDialog::Cancel | KDialog::Apply );
     setButtonFocus( KDialog::Ok );
+    setWindowTitle( i18n( "Launch Configurations" ) );
     button( KDialog::Apply )->setEnabled( false );
     
     setupUi( mainWidget() );
@@ -64,21 +65,16 @@ LaunchConfigurationDialog::LaunchConfigurationDialog(QWidget* parent): KDialog(p
     
     model = new LaunchConfigurationsModel( tree );
     tree->setModel( model );
-    tree->setIndentation( 5 );
     tree->setExpandsOnDoubleClick( true );
     tree->setSelectionBehavior( QAbstractItemView::SelectRows );
     tree->setSelectionMode( QAbstractItemView::SingleSelection );
     tree->setUniformRowHeights( true );
-    tree->setItemDelegateForColumn(0, new QItemDelegate( tree ) );
-    QItemDelegate* dlg = new QItemDelegate( tree );
-    dlg->setItemEditorFactory( new QItemEditorFactory() );
-    dlg->itemEditorFactory()->registerEditor( QVariant::String, new LaunchConfigurationTypeEditorCreator() );
-    tree->setItemDelegateForColumn( 1, dlg );
+    tree->setItemDelegate( new LaunchConfigurationModelDelegate() );
     
     connect( addConfig, SIGNAL(clicked()), this, SLOT(createConfiguration()));
     connect( deleteConfig, SIGNAL(clicked()), this, SLOT(deleteConfiguration()));
     
-    connect( tree->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), SLOT(selectedConfig(QItemSelection,QItemSelection)));
+    connect( tree->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), SLOT(selectionChanged(QItemSelection,QItemSelection)));
     QModelIndex idx = model->indexForConfig( Core::self()->runControllerInternal()->defaultLaunch() );
     kDebug() << "selecting index:" << idx;
     if( !idx.isValid() )
@@ -98,56 +94,115 @@ LaunchConfigurationDialog::LaunchConfigurationDialog(QWidget* parent): KDialog(p
     }
     tree->selectionModel()->select( QItemSelection( idx, idx ), QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows );
     tree->selectionModel()->setCurrentIndex( idx, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows );
-    tree->resizeColumnToContents( 0 );
     setInitialSize( QSize( 700, 500 ) );
     
+    // Unfortunately tree->resizeColumnToContents() only looks at the top-level 
+    // items, instead of all open ones. Hence we're calculating it ourselves like
+    // this:
+    // Take the selected index, check if it has childs, if so take the first child
+    // Then count the level by going up, then let the tree calculate the width
+    // for the selected or its first child index and add indentation*level
+    //
+    // If Qt Software ever fixes resizeColumnToContents, the following line
+    // can be enabled and the rest be removed
+    // tree->resizeColumnToContents( 0 );
+    int level = 0;
+    QModelIndex widthidx = idx;
+    if( model->rowCount( idx ) > 0 )
+    {
+        widthidx = idx.child( 0, 0 );
+    }
+    QModelIndex parentidx = widthidx.parent();
+    while( parentidx.isValid() )
+    {
+        level++;
+        parentidx = parentidx.parent();
+    }
+    int width = qMax( tree->columnWidth( 0 ), level*tree->indentation() + tree->indentation() + tree->sizeHintForIndex( widthidx ).width() );
+    tree->setColumnWidth( 0, width );
+    
     connect( this, SIGNAL(okClicked()), SLOT(saveConfig()) );
     connect( this, SIGNAL(applyClicked()), SLOT(saveConfig()) );
 }
 
-void LaunchConfigurationDialog::selectedConfig(QItemSelection selected, QItemSelection deselected )
+void LaunchConfigurationDialog::selectionChanged(QItemSelection selected, QItemSelection deselected )
 {
-    if( !deselected.indexes().isEmpty() && currentPageChanged )
+    if( !deselected.indexes().isEmpty() )
     {
         LaunchConfiguration* l = model->configForIndex( deselected.indexes().first() );
         if( l )
         {
-            disconnect(l, SIGNAL(nameChanged(QString)), this,  SLOT(updateNameLabel(QString)));
-            if( KMessageBox::questionYesNo( this, i18n("Selected Launch Configuration has unsaved changes. Do you want to save it?"), i18n("Unsaved Changes") ) == KMessageBox::Yes )
+            disconnect(l, SIGNAL(nameChanged(LaunchConfiguration*)), this,  SLOT(updateNameLabel(LaunchConfiguration*)));
+            if( currentPageChanged )
             {
-                saveConfig( deselected.indexes().first() );
-            } else {
-                LaunchConfigTypePage* tab = qobject_cast<LaunchConfigTypePage*>( stack->currentWidget() );
-                tab->setLaunchConfiguration( l );
-                currentPageChanged = false;
+                if( KMessageBox::questionYesNo( this, i18n("Selected Launch Configuration has unsaved changes. Do you want to save it?"), i18n("Unsaved Changes") ) == KMessageBox::Yes )
+                {
+                    saveConfig( deselected.indexes().first() );
+                } else {
+                    LaunchConfigPagesContainer* tab = qobject_cast<LaunchConfigPagesContainer*>( stack->currentWidget() );
+                    tab->setLaunchConfiguration( l );
+                    currentPageChanged = false;
+                }
             }
         }
     }
-    updateNameLabel("");
+    updateNameLabel("", "");
     if( !selected.indexes().isEmpty() )
     {
         LaunchConfiguration* l = model->configForIndex( selected.indexes().first() );
+        ILaunchMode* lm = model->modeForIndex( selected.indexes().first() );
         if( l )
         {
-            //TODO: enable removal button
-            LaunchConfigurationType* type = l->type();
-            LaunchConfigTypePage* tab;
-            if( typeWidgets.contains( type ) )
+            updateNameLabel( l );
+            tree->expand( model->indexForConfig( l ) );
+            connect( l, SIGNAL(nameChanged(LaunchConfiguration*)), SLOT(updateNameLabel(LaunchConfiguration*)) );
+            if( lm )
             {
-                tab = typeWidgets.value( type );
-            } else
+                ILauncher* launcher = l->type()->launcherForId( l->launcherForMode( lm->id() ) );
+                if( launcher )
+                {
+                    LaunchConfigPagesContainer* tab;
+                    if( launcherWidgets.contains( launcher ) )
+                    {
+                        tab = launcherWidgets.value( launcher );
+                    } else
+                    {
+                        tab = new LaunchConfigPagesContainer( launcher->configPages(), stack );
+                        connect( tab, SIGNAL(changed()), SLOT(pageChanged()) );
+                        stack->addWidget( tab );
+                    }
+                    tab->setLaunchConfiguration( l );
+                    stack->setCurrentWidget( tab );
+                    updateNameLabel( l );
+                    addConfig->setEnabled( false );
+                    deleteConfig->setEnabled( false );
+                } else
+                {
+                    addConfig->setEnabled( false );
+                    deleteConfig->setEnabled( false );
+                    stack->setCurrentIndex( 0 );
+                }
+            } else 
             {
-                tab = new LaunchConfigTypePage( type, stack );
-                connect( tab, SIGNAL(changed()), SLOT(pageChanged()) );
-                stack->addWidget( tab );
+                //TODO: enable removal button
+                LaunchConfigurationType* type = l->type();
+                LaunchConfigPagesContainer* tab;
+                if( typeWidgets.contains( type ) )
+                {
+                    tab = typeWidgets.value( type );
+                } else
+                {
+                    tab = new LaunchConfigPagesContainer( type->configPages(), stack );
+                    connect( tab, SIGNAL(changed()), SLOT(pageChanged()) );
+                    stack->addWidget( tab );
+                }
+                kDebug() << "created pages, setting config up";
+                tab->setLaunchConfiguration( l );
+                stack->setCurrentWidget( tab );
+                
+                addConfig->setEnabled( true );
+                deleteConfig->setEnabled( true );
             }
-            tab->setLaunchConfiguration( l );
-            updateNameLabel( l->name() );
-            connect( l, SIGNAL(nameChanged(QString)), SLOT(updateNameLabel(QString)) );
-            stack->setCurrentWidget( tab );
-            
-            addConfig->setEnabled( true );
-            deleteConfig->setEnabled( true );
         } else 
         {
             addConfig->setEnabled( true );
@@ -164,7 +219,8 @@ void LaunchConfigurationDialog::selectedConfig(QItemSelection selected, QItemSel
 
 void LaunchConfigurationDialog::saveConfig( const QModelIndex& idx )
 {
-    LaunchConfigTypePage* tab = qobject_cast<LaunchConfigTypePage*>( stack->currentWidget() );
+    Q_UNUSED( idx );
+    LaunchConfigPagesContainer* tab = qobject_cast<LaunchConfigPagesContainer*>( stack->currentWidget() );
     if( tab )
     {
         tab->save();
@@ -198,10 +254,26 @@ void LaunchConfigurationDialog::deleteConfiguration()
     }
 }
 
-
-void LaunchConfigurationDialog::updateNameLabel( const QString& name )
+void LaunchConfigurationDialog::updateNameLabel( LaunchConfiguration* l )
 {
-    configName->setText( i18n("<b>%1</b>", name ) );
+    if( l )
+    {
+        updateNameLabel( l->name(), l->project() ? l->project()->name() : "" );
+    } else
+    {
+        updateNameLabel( "", "" );
+    }
+}
+
+void LaunchConfigurationDialog::updateNameLabel( const QString& name, const QString& project )
+{
+    if( project.isEmpty() )
+    {
+        configName->setText( i18n("<b>%1</b>", name ) );
+    } else
+    {
+        configName->setText( i18n("<b>%1</b> (%2)", name, project ) );
+    }
 }
 
 
@@ -225,39 +297,62 @@ void LaunchConfigurationDialog::createConfiguration()
 
 LaunchConfigurationsModel::LaunchConfigurationsModel(QObject* parent): QAbstractItemModel(parent)
 {
-    TreeItem* global = new TreeItem;
+    GenericPageItem* global = new GenericPageItem;
+    global->text = i18n("Global");
     global->row = 0;
     topItems << global;
     foreach( IProject* p, Core::self()->projectController()->projects() )
     {
-        TreeItem* t = new TreeItem;
+        ProjectItem* t = new ProjectItem;
         t->project = p;
         t->row = topItems.count();
         topItems << t;
     }
     foreach( LaunchConfiguration* l, Core::self()->runControllerInternal()->launchConfigurations() )
     {
-        TreeItem* t = new TreeItem;
-        t->launch = l;
-        TreeItem* parent;
-        if( l->project() ) {
-            parent = findItemForProject( l->project() );
-        } else {
-            parent = topItems.at(0);
-        }
-        t->parent = parent;
-        t->row = parent->children.count();
-        parent->children.append( t );
+        addItemForLaunchConfig( l );
     }    
 }
 
-LaunchConfigurationsModel::TreeItem* LaunchConfigurationsModel::findItemForProject( IProject* p )
+void LaunchConfigurationsModel::addItemForLaunchConfig( LaunchConfiguration* l )
+{
+    LaunchItem* t = new LaunchItem;
+    t->launch = l;
+    TreeItem* parent;
+    if( l->project() ) {
+        parent = findItemForProject( l->project() );
+    } else {
+        parent = topItems.at(0);
+    }
+    t->parent = parent;
+    t->row = parent->children.count();
+    parent->children.append( t );
+    QSet<QString> modes;
+    foreach( ILauncher* launcher, l->type()->launchers() )
+    {
+        foreach( const QString& mode, launcher->supportedModes() )
+        {
+            if( !modes.contains( mode ) && launcher->configPages().count() > 0 )
+            {
+                modes.insert( mode );
+                LaunchModeItem* lmi = new LaunchModeItem;
+                lmi->mode = Core::self()->runController()->launchModeForId( mode );
+                lmi->parent = t;
+                lmi->row = t->children.count();
+                t->children.append( lmi );
+            }
+        }
+    }
+}
+
+LaunchConfigurationsModel::ProjectItem* LaunchConfigurationsModel::findItemForProject( IProject* p )
 {
     foreach( TreeItem* t, topItems )
     {
-        if( t->project == p ) 
+        ProjectItem* pi = dynamic_cast<ProjectItem*>( t );
+        if( pi && pi->project == p ) 
         {
-            return t;
+            return pi;
         }
     }
     Q_ASSERT(false);
@@ -266,6 +361,7 @@ LaunchConfigurationsModel::TreeItem* LaunchConfigurationsModel::findItemForProje
 
 int LaunchConfigurationsModel::columnCount(const QModelIndex& parent) const
 {
+    Q_UNUSED( parent );
     return 2;
 }
 
@@ -277,49 +373,75 @@ QVariant LaunchConfigurationsModel::data(const QModelIndex& index, int role) con
         switch( role ) 
         {
             case Qt::DisplayRole:
-                if( index.column() == 0 )
-                {
-                    if( t->launch ) 
-                    {
-                        return t->launch->name();
-                    } else if( t->project )
-                    {
-                        return t->project->name();
-                    } else
-                    {
-                        return i18n("Global");
-                    }
-                } else if( index.column() == 1 )
-                {
-                    if( t->launch )
-                    {
-                        return t->launch->type()->name();
-                    } else
-                    {
-                        return "";
-                    }
-                }
-                break;
-            case Qt::DecorationRole:
-                if( index.column() == 0 && t->launch )
-                {
-                    return t->launch->type()->icon();
-                } else if( index.column() == 1 )
-                {
-                    return KIcon();
-                }
-            case Qt::EditRole:
-                if( index.parent().isValid() )
+            {
+                LaunchItem* li = dynamic_cast<LaunchItem*>( t );
+                if( li ) 
                 {
                     if( index.column() == 0 )
                     {
-                        return t->launch->name();
-                    } else if ( index.column() == 1 )
+                        return li->launch->name();   
+                    } else if( index.column() == 1 )
                     {
-                        return t->launch->type()->id();
+                        return li->launch->type()->name();
+                    }
+                }
+                ProjectItem* pi = dynamic_cast<ProjectItem*>( t );
+                if( pi && index.column() == 0 ) 
+                {
+                    return pi->project->name();
+                }
+                GenericPageItem* gpi = dynamic_cast<GenericPageItem*>( t );
+                if( gpi && index.column() == 0 )
+                {
+                    return gpi->text;
+                }
+                LaunchModeItem* lmi = dynamic_cast<LaunchModeItem*>( t );
+                if( lmi )
+                {
+                    if( index.column() == 0 )
+                    {
+                        return lmi->mode->name();
+                    } else if( index.column() == 1 )
+                    {
+                        LaunchConfiguration* l = configForIndex( index );
+                        return l->type()->launcherForId( l->launcherForMode( lmi->mode->id() ) )->name();
                     }
                 }
                 break;
+            }
+            case Qt::DecorationRole:
+            {
+                LaunchItem* li = dynamic_cast<LaunchItem*>( t );
+                if( index.column() == 0 && li )
+                {
+                    return li->launch->type()->icon();
+                }
+                LaunchModeItem* lmi = dynamic_cast<LaunchModeItem*>( t );
+                if( lmi && index.column() == 0 )
+                {
+                    return lmi->mode->icon();
+                }
+            }
+            case Qt::EditRole:
+            {
+                LaunchItem* li = dynamic_cast<LaunchItem*>( t );
+                if( li )
+                {
+                    if( index.column() == 0 )
+                    {
+                        return li->launch->name();
+                    } else if ( index.column() == 1 )
+                    {
+                        return li->launch->type()->id();
+                    }
+                }
+                LaunchModeItem* lmi = dynamic_cast<LaunchModeItem*>( t );
+                if( lmi && index.column() == 1  )
+                {
+                    return configForIndex( index )->launcherForMode( lmi->mode->id() );
+                }
+                break;
+            }
             default:
                 break;
         }
@@ -397,7 +519,7 @@ Qt::ItemFlags LaunchConfigurationsModel::flags(const QModelIndex& index) const
         && index.column() < columnCount( QModelIndex() ) ) 
     {
         TreeItem* t = static_cast<TreeItem*>( index.internalPointer() );
-        if( t && t->launch )
+        if( t && ( dynamic_cast<LaunchItem*>( t ) || ( dynamic_cast<LaunchModeItem*>( t ) && index.column() == 1 ) ) )
         {
             return Qt::ItemFlags( Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsEditable );
         } else if( t )
@@ -414,15 +536,27 @@ bool LaunchConfigurationsModel::setData(const QModelIndex& index, const QVariant
     {
         if( index.row() >= 0 && index.row() < rowCount( index.parent() ) ) 
         {
-            TreeItem* t = static_cast<TreeItem*>( index.internalPointer() );
+            LaunchItem* t = dynamic_cast<LaunchItem*>( static_cast<TreeItem*>( index.internalPointer() ) );
             if( t )
             {
                 if( index.column() == 0 )
                 {
                     t->launch->setName( value.toString() );
+                    return true;
                 } else if( index.column() == 1 )
                 {
                     t->launch->setType( value.toString() );
+                    return true;
+                }
+            }
+            LaunchModeItem* lmi = dynamic_cast<LaunchModeItem*>( static_cast<TreeItem*>( index.internalPointer() ) );
+            if( lmi )
+            {
+                if( index.column() == 1 )
+                {
+                    LaunchConfiguration* l = configForIndex( index );
+                    l->setLauncherForMode( lmi->mode->id(), value.toString() );
+                    return true;
                 }
             }
         }
@@ -430,17 +564,38 @@ bool LaunchConfigurationsModel::setData(const QModelIndex& index, const QVariant
     return false;
 }
 
-LaunchConfiguration* LaunchConfigurationsModel::configForIndex(const QModelIndex& idx )
+ILaunchMode* LaunchConfigurationsModel::modeForIndex( const QModelIndex& idx ) const
 {
     if( idx.isValid() )
     {
-        TreeItem* item = static_cast<TreeItem*>( idx.internalPointer() );
-        return item->launch;
+        LaunchModeItem* item = dynamic_cast<LaunchModeItem*>( static_cast<TreeItem*>( idx.internalPointer() ) );
+        if( item )
+        {
+            return item->mode;
+        }
     }
     return 0;
 }
 
-QModelIndex LaunchConfigurationsModel::indexForConfig( LaunchConfiguration* l )
+LaunchConfiguration* LaunchConfigurationsModel::configForIndex(const QModelIndex& idx ) const
+{
+    if( idx.isValid() )
+    {
+        LaunchItem* item = dynamic_cast<LaunchItem*>( static_cast<TreeItem*>( idx.internalPointer() ) );
+        if( item )
+        {
+            return item->launch;
+        } 
+        LaunchModeItem* lmitem = dynamic_cast<LaunchModeItem*>( static_cast<TreeItem*>( idx.internalPointer() ) );
+        if( lmitem ) 
+        {
+            return dynamic_cast<LaunchItem*>( lmitem->parent )->launch;
+        }
+    }
+    return 0;
+}
+
+QModelIndex LaunchConfigurationsModel::indexForConfig( LaunchConfiguration* l ) const
 {
     if( l )
     {
@@ -449,7 +604,8 @@ QModelIndex LaunchConfigurationsModel::indexForConfig( LaunchConfiguration* l )
         {
             foreach( TreeItem* t, topItems )
             {
-                if( t->project && t->project == l->project() )
+                ProjectItem* pi = dynamic_cast<ProjectItem*>( t );
+                if( pi && pi->project == l->project() )
                 {
                     tparent = t;
                     break;
@@ -461,7 +617,8 @@ QModelIndex LaunchConfigurationsModel::indexForConfig( LaunchConfiguration* l )
         {
             foreach( TreeItem* c, tparent->children )
             {
-                if( c->launch && c->launch == l )
+                LaunchItem* li = dynamic_cast<LaunchItem*>( c );
+                if( li->launch && li->launch == l )
                 {
                     return index( c->row, 0, index( tparent->row, 0, QModelIndex() ) );
                 }
@@ -474,7 +631,9 @@ QModelIndex LaunchConfigurationsModel::indexForConfig( LaunchConfiguration* l )
 
 void LaunchConfigurationsModel::deleteConfiguration( const QModelIndex& index )
 {
-    TreeItem* t = static_cast<TreeItem*>( index.internalPointer() );
+    LaunchItem* t = dynamic_cast<LaunchItem*>( static_cast<TreeItem*>( index.internalPointer() ) );
+    if( !t )
+        return;
     beginRemoveRows( parent( index ), index.row(), index.row() );
     t->parent->children.removeAll( t );
     Core::self()->runControllerInternal()->removeLaunchConfiguration( t->launch );
@@ -484,17 +643,22 @@ void LaunchConfigurationsModel::deleteConfiguration( const QModelIndex& index )
 void LaunchConfigurationsModel::createConfiguration(const QModelIndex& parent )
 {
     TreeItem* t = static_cast<TreeItem*>( parent.internalPointer() );
+    ProjectItem* ti = dynamic_cast<ProjectItem*>( t );
     if( parent.isValid() && t && !Core::self()->runController()->launchConfigurationTypes().isEmpty() )
     {
         KConfigGroup launchGroup;
-        if( t->project )
+        if( ti )
         {
-            launchGroup = t->project->projectConfiguration()->group( RunController::LaunchConfigurationsGroup );
+            launchGroup = ti->project->projectConfiguration()->group( RunController::LaunchConfigurationsGroup );
             
-        } else 
+        } else if( !t->parent )
         {
             launchGroup = Core::self()->activeSession()->config()->group( RunController::LaunchConfigurationsGroup );
             
+        } else 
+        {
+            kWarning() << "Expected project or global item, but didn't find either";
+            return;
         }
         beginInsertRows( parent, rowCount( parent ), rowCount( parent ) );
         QStringList configs = launchGroup.readEntry( RunController::LaunchConfigurationsListEntry, QStringList() );
@@ -513,166 +677,51 @@ void LaunchConfigurationsModel::createConfiguration(const QModelIndex& parent )
         configs << groupName;
         launchGroup.writeEntry( RunController::LaunchConfigurationsListEntry, configs );
         launchGroup.sync();
-        LaunchConfiguration* l = new LaunchConfiguration( launchConfigGroup, t->project ? t->project : 0 );
+        LaunchConfiguration* l = new LaunchConfiguration( launchConfigGroup, ( ti ? ti->project : 0 ) );
         l->setLauncherForMode( type->launchers().at( 0 )->supportedModes().at(0), type->launchers().at( 0 )->id() );
         Core::self()->runControllerInternal()->addLaunchConfiguration( l );
-        TreeItem* item = new TreeItem;
-        item->launch = l;
-        item->parent = t;
-        item->row = t->children.count();
-        t->children.append( item );
+        addItemForLaunchConfig( l );
         endInsertRows();
     }
 }
 
-LaunchConfigurationTypeComboBox::LaunchConfigurationTypeComboBox( QWidget* parent )
-    : KComboBox( parent )
+LaunchConfigPagesContainer::LaunchConfigPagesContainer( const QList<LaunchConfigurationPageFactory*>& factories, QWidget* parent ) 
+    : QWidget(parent)
 {
-    foreach( LaunchConfigurationType* t, Core::self()->runControllerInternal()->launchConfigurationTypes() )
+    setLayout( new QVBoxLayout( this ) );
+    QWidget* parentwidget = this;
+    KTabWidget* tab = 0;
+    if( factories.count() > 1 )
     {
-        addItem( t->name(), t->id() );
+        tab = new KTabWidget( this );
+        parentwidget = tab;
+        layout()->addWidget( tab );
     }
-}
-
-QVariant LaunchConfigurationTypeComboBox::currentData() const
-{
-    return itemData( currentIndex() );
-}
-
-void LaunchConfigurationTypeComboBox::changeCurrentIndex( int idx )
-{
-    emit currentDataChanged( itemData( idx ) );
-}
-
-void LaunchConfigurationTypeComboBox::setCurrentData( const QVariant& data )
-{
-    setCurrentIndex( findData( data ) );
-}
-
-LaunchConfigurationTypeEditorCreator::LaunchConfigurationTypeEditorCreator()
-{
-}
-
-QWidget* LaunchConfigurationTypeEditorCreator::createWidget(QWidget* parent) const
-{
-    LaunchConfigurationTypeComboBox* cb = new LaunchConfigurationTypeComboBox( parent );
-    cb->setFrame( false );
-    return cb;
-}
-
-QByteArray LaunchConfigurationTypeEditorCreator::valuePropertyName() const
-{
-    return "currentData";
-}
-
-LaunchConfigTypePage::LaunchConfigTypePage( LaunchConfigurationType* type, QWidget* parent ) 
-    : QWidget(parent), ui( new Ui::LaunchConfigTypePage )
-{
-    ui->setupUi( this );
-    ui->launcher->setEnabled( false );
-    foreach( ILauncher* l, type->launchers() )
+    foreach( LaunchConfigurationPageFactory* fac, factories )
     {
-        foreach( const QString& s, l->supportedModes() )
-        {
-            ILaunchMode* m = Core::self()->runControllerInternal()->launchModeForId( s );
-            if( m )
-            {
-                int idx = ui->mode->findData( m->id() );
-                if( idx == -1 )
-                {
-                    ui->mode->addItem( m->icon(), m->name(), m->id() );
-                    launchersForModes.insert( m->id(), QStringList() << l->id() );
-                } else
-                {
-                    launchersForModes[ m->id() ] << l->id();
-                }
-            } else
-            {
-                kWarning() << "Ooops, config type" << type->id() << "provides a launcher with an unknown launch mode:" << s;
-                return;
-            }
-        }
-    }
-    
-    foreach( LaunchConfigurationPageFactory* fac, type->configPages() )
-    {
-        LaunchConfigurationPage* page = fac->createWidget( ui->tab );
+        LaunchConfigurationPage* page = fac->createWidget( parentwidget );
+        pages.append( page );
         connect( page, SIGNAL(changed()), SIGNAL(changed()) );
-        ui->tab->addTab( page, page->icon(), page->title() );
-    }
-    
-    connect( ui->mode, SIGNAL(currentIndexChanged(int)), SLOT(changeMode(int)) );
-    connect( ui->launcher, SIGNAL(currentIndexChanged(int)), SLOT(changeLauncher(int)) );
-    connect( ui->launcher, SIGNAL(currentIndexChanged(int)), SIGNAL(changed()) );
-    
-}
-
-void LaunchConfigTypePage::changeLauncher( int idx )
-{
-    foreach( LaunchConfigurationPage* p, launcherPages )
-    {
-        ui->tab->removePage( p );
-        delete p;
-    }
-    launcherPages.clear();
-    ILauncher* l = config->type()->launcherForId( ui->launcher->itemData( idx ).toString() );
-    if( l )
-    {
-        config->setLauncherForMode( ui->mode->itemData( ui->mode->currentIndex() ).toString(), ui->launcher->itemData( idx ).toString() );
-        foreach( LaunchConfigurationPageFactory* fac, l->configPages() )
+        if( tab ) {
+            tab->addTab( page, page->icon(), page->title() );
+        } else 
         {
-            LaunchConfigurationPage* page = fac->createWidget( ui->tab );
-            connect( page, SIGNAL(changed()), SIGNAL(changed()) );
-            ui->tab->addTab( page, page->icon(), page->title() );
-            page->loadFromConfiguration( config->config() );
-            launcherPages << page;
+            layout()->addWidget( page );
         }
     }
-    
 }
 
-void LaunchConfigTypePage::changeMode( int idx )
-{
-    QString id = ui->mode->itemData( idx ).toString();
-    QStringList launchers = launchersForModes[id];
-    
-    if( launchers.count() > 1 )
-    {
-        ui->launcher->setEnabled( true );
-    } else
-    {
-        ui->launcher->setEnabled( false );
-    }
-    bool b = ui->launcher->blockSignals( true );
-    ui->launcher->clear();
-    foreach( const QString& lid, launchers )
-    {
-        ui->launcher->addItem( config->type()->launcherForId( lid )->name(), lid );
-    }
-    int lidx = 0;
-    if( launchers.contains( config->launcherForMode( id ) ) )
-    {
-        lidx = ui->launcher->findData( config->launcherForMode( id ) );
-    } 
-    ui->launcher->setCurrentIndex( lidx );
-    changeLauncher( lidx );
-    ui->launcher->blockSignals( b );
-}
-
-void LaunchConfigTypePage::setLaunchConfiguration( KDevelop::LaunchConfiguration* l )
+void LaunchConfigPagesContainer::setLaunchConfiguration( KDevelop::LaunchConfiguration* l )
 {
     config = l;
-    changeMode( ui->mode->findData( "execute" ) );
-    changeLauncher( ui->launcher->findData( config->launcherForMode( "execute" ) ) );
-    foreach( LaunchConfigurationPage* p, ui->tab->findChildren<LaunchConfigurationPage*>() )
+    foreach( LaunchConfigurationPage* p, pages )
     {
-        p->loadFromConfiguration( config->config() );
+        p->loadFromConfiguration( config->config(), config->project() );
     }
 }
 
-void LaunchConfigTypePage::save()
+void LaunchConfigPagesContainer::save()
 {
-    QList<LaunchConfigurationPage*> pages = ui->tab->findChildren<LaunchConfigurationPage*>();
     foreach( LaunchConfigurationPage* p, pages )
     {
         p->saveToConfiguration( config->config() );
@@ -681,6 +730,65 @@ void LaunchConfigTypePage::save()
 }
 
 
+QWidget* LaunchConfigurationModelDelegate::createEditor ( QWidget* parent, const QStyleOptionViewItem& option, const QModelIndex& index ) const
+{
+    const LaunchConfigurationsModel* model = dynamic_cast<const LaunchConfigurationsModel*>( index.model() );
+    ILaunchMode* mode = model->modeForIndex( index );
+    LaunchConfiguration* config = model->configForIndex( index );
+    if( index.column() == 1 && mode && config ) 
+    {
+        KComboBox* box = new KComboBox( parent );
+        QList<ILauncher*> launchers = config->type()->launchers();
+        for( QList<ILauncher*>::const_iterator it = launchers.constBegin(); it != launchers.constEnd(); it++ )
+        {
+            if( ((*it)->supportedModes().contains( mode->id() ) ) )
+            {
+                box->addItem( (*it)->name(), (*it)->id() );
+            }
+        }
+        return box;
+    } else if( !mode && config && index.column() == 1 )
+    {
+        KComboBox* box = new KComboBox( parent );
+        QList<LaunchConfigurationType*> types = Core::self()->runController()->launchConfigurationTypes();
+        for( QList<LaunchConfigurationType*>::const_iterator it = types.begin(); it != types.end(); it++ )
+        {
+            box->addItem( (*it)->name(), (*it)->id() );
+        }
+        return box;
+    }
+    return QItemDelegate::createEditor ( parent, option, index );
+}
+
+LaunchConfigurationModelDelegate::LaunchConfigurationModelDelegate()
+{
+}
+
+void LaunchConfigurationModelDelegate::setEditorData ( QWidget* editor, const QModelIndex& index ) const
+{
+    const LaunchConfigurationsModel* model = dynamic_cast<const LaunchConfigurationsModel*>( index.model() );
+    LaunchConfiguration* config = model->configForIndex( index );
+    if( index.column() == 1 && config ) 
+    {
+        KComboBox* box = qobject_cast<KComboBox*>( editor );
+        box->setCurrentIndex( box->findData( index.data( Qt::EditRole ) ) );
+    }
+    QItemDelegate::setEditorData ( editor, index );
+}
+
+void LaunchConfigurationModelDelegate::setModelData ( QWidget* editor, QAbstractItemModel* model, const QModelIndex& index ) const
+{
+    LaunchConfigurationsModel* lmodel = dynamic_cast<LaunchConfigurationsModel*>( model );
+    LaunchConfiguration* config = lmodel->configForIndex( index );
+    if( index.column() == 1 && config ) 
+    {
+        KComboBox* box = qobject_cast<KComboBox*>( editor );
+        lmodel->setData( index, box->itemData( box->currentIndex() ) );
+    }
+    QItemDelegate::setModelData ( editor, model, index );
+}
+
+
 }
 
 #include "launchconfigurationdialog.moc"
