@@ -137,11 +137,42 @@ void MainWindow::saveSettings()
     Sublime::MainWindow::saveSettings();
 }
 
+
+void MainWindow::configureShortcuts()
+{
+    ///Workaround for a problem with the actions: Always start the shortcut-configuration in the first mainwindow, then propagate the updated
+    ///settings into the other windows
+    Core::self()->uiControllerInternal()->mainWindows()[0]->guiFactory()->configureShortcuts();
+
+    QMap<QString, QKeySequence> shortcuts;
+    foreach(KXMLGUIClient* client, Core::self()->uiControllerInternal()->mainWindows()[0]->guiFactory()->clients()) {
+        foreach(QAction* action, client->actionCollection()->actions()) {
+            if(!action->objectName().isEmpty()) {
+                shortcuts[action->objectName()] = action->shortcut();
+            }
+        }
+    }
+    
+    for(int a = 1; a < Core::self()->uiControllerInternal()->mainWindows().size(); ++a) {
+        foreach(KXMLGUIClient* client, Core::self()->uiControllerInternal()->mainWindows()[a]->guiFactory()->clients()) {
+            foreach(QAction* action, client->actionCollection()->actions()) {
+                kDebug() << "transferring setting shortcut for" << action->objectName();
+                if(shortcuts.contains(action->objectName())) {
+                    action->setShortcut(shortcuts[action->objectName()]);
+                }
+            }
+        }
+    }
+}
+
 void MainWindow::initialize()
 {
-    setupGUI( KXmlGuiWindow::Keys | KXmlGuiWindow::ToolBar | KXmlGuiWindow::Create | KXmlGuiWindow::Save );
+    KStandardAction::keyBindings(this, SLOT(configureShortcuts()), actionCollection());
+    setupGUI( KXmlGuiWindow::ToolBar | KXmlGuiWindow::Create | KXmlGuiWindow::Save );
+    
     Core::self()->partController()->addManagedTopLevelWidget(this);
     kDebug() << "Adding plugin-added connection";
+    
     connect( Core::self()->pluginController(), SIGNAL(pluginLoaded(KDevelop::IPlugin*)),
              d, SLOT(addPlugin(KDevelop::IPlugin*)));
     connect( Core::self()->partController(), SIGNAL(activePartChanged(KParts::Part*)),
@@ -151,6 +182,10 @@ void MainWindow::initialize()
     connect(Core::self()->documentController(), SIGNAL(documentActivated(KDevelop::IDocument*)), SLOT(documentActivated(KDevelop::IDocument*)));
     connect(Core::self()->documentController(), SIGNAL(documentStateChanged(KDevelop::IDocument*)), SLOT(documentStateChanged(KDevelop::IDocument*)));
     connect(Core::self()->documentController(), SIGNAL(documentClosed(KDevelop::IDocument*)), SLOT(documentClosed(KDevelop::IDocument*)));
+    
+    foreach(IPlugin* plugin, Core::self()->pluginController()->loadedPlugins())
+        d->addPlugin(plugin);
+    
     guiFactory()->addClient(Core::self()->debugControllerInternal());
     d->setupGui();
 }
@@ -209,6 +244,12 @@ void MainWindow::setupAreaSelector()
     d->setupAreaSelector();
 }
 
+
+void MainWindow::showErrorMessage(const QString& message, int timeout)
+{
+    d->showErrorMessage(message, timeout);
+}
+
 }
 
 #include "mainwindow.moc"
