@@ -105,6 +105,9 @@ public:
 
     QHash<KJob*, KAction*> jobs;
     KActionMenu* stopAction;
+    KAction* profileAction;
+    KAction* runAction;
+    KAction* dbgAction;
     KSelectAction* currentTargetAction;
     QMap<QString,LaunchConfigurationType*> launchConfigurationTypes;
     QList<LaunchConfiguration*> launchConfigurations;
@@ -127,6 +130,12 @@ public:
             grp.sync();
         }
     }
+    void enableLaunchActions()
+    {
+        runAction->setEnabled( !launchConfigurations.isEmpty() );
+        profileAction->setEnabled( !launchConfigurations.isEmpty() );
+        dbgAction->setEnabled( !launchConfigurations.isEmpty() );
+    }
     void configureLaunches()
     {
         LaunchConfigurationDialog dlg;
@@ -237,7 +246,6 @@ RunController::RunController(QObject *parent)
     d->q = this;
     d->delegate = new RunDelegate(this);
     d->launchChangeMapper = new QSignalMapper( this );
-    connect(d->launchChangeMapper, SIGNAL(mapped(int)), SLOT(launchChanged(int)) );
 
     if(!(Core::self()->setupFlags() & Core::NoUi)) {
         // Note that things like registerJob() do not work without the actions, it'll simply crash.
@@ -245,9 +253,8 @@ RunController::RunController(QObject *parent)
     }
 }
 
-void KDevelop::RunController::launchChanged( int i )
+void KDevelop::RunController::launchChanged( LaunchConfiguration* l )
 {
-    LaunchConfiguration* l = d->launchConfigurations.at( i );
     foreach( QAction* a, d->currentTargetAction->actions() )
     {
         if( static_cast<LaunchConfiguration*>( qVariantValue<void*>( a->data() ) ) == l )
@@ -280,7 +287,12 @@ void RunController::initialize()
     connect(Core::self()->projectController(), SIGNAL(projectConfigurationChanged(KDevelop::IProject*)),
              this, SLOT(slotRefreshProject(KDevelop::IProject*)));
 
-    d->updateCurrentLaunchAction();
+    if( (Core::self()->setupFlags() & Core::NoUi) == 0 )
+    {
+        // Only do this in GUI mode
+        d->updateCurrentLaunchAction();
+        d->enableLaunchActions();
+    }
 }
 
 KJob* RunController::execute(const QString& runMode, LaunchConfiguration* run)
@@ -334,30 +346,33 @@ void RunController::setupActions()
     action->setWhatsThis(i18n("<p>Opens a dialog to setup new launch configurations, or to change the existing ones.</p>"));
     connect(action, SIGNAL(triggered(bool)), SLOT(configureLaunches()));
 
-    action = new KAction( KIcon("system-run"), i18n("Execute Launch"), this);
-    action->setShortcut(Qt::SHIFT + Qt::Key_F9);
-    action->setToolTip(i18n("Execute current Launch"));
-    action->setStatusTip(i18n("Execute current Launch"));
-    action->setWhatsThis(i18n("<b>Execute Launch</b><p>Executes the target or the program specified in currently active launch configuration.</p>"));
-    ac->addAction("run_execute", action);
-    connect(action, SIGNAL(triggered(bool)), this, SLOT(slotExecute()));
+    d->runAction = new KAction( KIcon("system-run"), i18n("Execute Launch"), this);
+    d->runAction->setIconText( i18nc("Short text for 'Execute Launch' used in the toolbar", "Execute") );
+    d->runAction->setShortcut(Qt::SHIFT + Qt::Key_F9);
+    d->runAction->setToolTip(i18n("Execute current Launch"));
+    d->runAction->setStatusTip(i18n("Execute current Launch"));
+    d->runAction->setWhatsThis(i18n("<b>Execute Launch</b><p>Executes the target or the program specified in currently active launch configuration.</p>"));
+    ac->addAction("run_execute", d->runAction);
+    connect(d->runAction, SIGNAL(triggered(bool)), this, SLOT(slotExecute()));
     
-    action = new KAction( KIcon("dbgrun"), i18n("Debug Launch"), this);
-    action->setShortcut(Qt::Key_F9);
-    action->setToolTip(i18n("Debug current Launch"));
-    action->setStatusTip(i18n("Debug current Launch"));
-    action->setWhatsThis(i18n("<b>Debug Launch</b><p>Executes the target or the program specified in currently active launch configuration inside a Debugger.</p>"));
-    ac->addAction("run_debug", action);
-    connect(action, SIGNAL(triggered(bool)), this, SLOT(slotDebug()));
+    d->dbgAction = new KAction( KIcon("dbgrun"), i18n("Debug Launch"), this);
+    d->dbgAction->setShortcut(Qt::Key_F9);
+    d->dbgAction->setIconText( i18nc("Short text for 'Debug Launch' used in the toolbar", "Debug") );
+    d->dbgAction->setToolTip(i18n("Debug current Launch"));
+    d->dbgAction->setStatusTip(i18n("Debug current Launch"));
+    d->dbgAction->setWhatsThis(i18n("<b>Debug Launch</b><p>Executes the target or the program specified in currently active launch configuration inside a Debugger.</p>"));
+    ac->addAction("run_debug", d->dbgAction);
+    connect(d->dbgAction, SIGNAL(triggered(bool)), this, SLOT(slotDebug()));
     
-    action = new KAction( KIcon(""), i18n("Profile Launch"), this);
-    action->setToolTip(i18n("Profile current Launch"));
-    action->setStatusTip(i18n("Profile current Launch"));
-    action->setWhatsThis(i18n("<b>Profile Launch</b><p>Executes the target or the program specified in currently active launch configuration inside a Profiler.</p>"));
-    ac->addAction("run_profile", action);
-    connect(action, SIGNAL(triggered(bool)), this, SLOT(slotProfile()));
+    d->profileAction = new KAction( KIcon(""), i18n("Profile Launch"), this);
+    d->profileAction->setToolTip(i18n("Profile current Launch"));
+    d->profileAction->setStatusTip(i18n("Profile current Launch"));
+    d->profileAction->setWhatsThis(i18n("<b>Profile Launch</b><p>Executes the target or the program specified in currently active launch configuration inside a Profiler.</p>"));
+    ac->addAction("run_profile", d->profileAction);
+    connect(d->profileAction, SIGNAL(triggered(bool)), this, SLOT(slotProfile()));
 
     action = d->stopAction = new KActionMenu( KIcon("dialog-close"), i18n("Stop Jobs"), this);
+    action->setIconText(i18nc("Short text for 'Stop Jobs' used in the toolbar", "Stop"));
     action->setShortcut(Qt::Key_Escape);
     action->setToolTip(i18n("Stop all currently running jobs"));
     action->setWhatsThis(i18n("<b>Stop Jobs</b><p>Requests that all running jobs are stopped.</p>"));
@@ -397,6 +412,7 @@ void KDevelop::RunController::slotProjectClosing(KDevelop::IProject * project)
                 d->currentTargetAction->actions().first()->setChecked(true);
         }
     }
+    d->enableLaunchActions();
 }
 
 void KDevelop::RunController::slotRefreshProject(KDevelop::IProject* project)
@@ -605,6 +621,7 @@ void KDevelop::RunController::addLaunchConfiguration(KDevelop::LaunchConfigurati
     {
         d->addLaunchAction( l );
         d->launchConfigurations << l;
+        d->enableLaunchActions();
         if( !d->currentTargetAction->currentAction() )
         {
             if( !d->currentTargetAction->actions().isEmpty() )
@@ -612,8 +629,7 @@ void KDevelop::RunController::addLaunchConfiguration(KDevelop::LaunchConfigurati
                 d->currentTargetAction->actions().first()->setChecked( true );
             }
         }
-        d->launchChangeMapper->setMapping( l, d->launchConfigurations.count() -1 );
-        connect( l, SIGNAL(nameChanged(QString)), d->launchChangeMapper, SLOT(map()) );
+        connect( l, SIGNAL(nameChanged(LaunchConfiguration*)), SLOT(launchChanged(LaunchConfiguration*)) );
     }    
 }
 
@@ -647,6 +663,8 @@ void KDevelop::RunController::removeLaunchConfiguration(KDevelop::LaunchConfigur
 
     d->launchConfigurations.removeAll( l );
     
+    d->enableLaunchActions();
+    
     delete l;
 }
 
