@@ -65,6 +65,7 @@ EnvironmentWidget::EnvironmentWidget( QWidget *parent )
     connect( ui.setAsDefaultBtn, SIGNAL( clicked() ), SIGNAL( changed() ) );
     connect( ui.activeCombo, SIGNAL(currentIndexChanged(int)),
              SLOT( activeGroupChanged(int)) );
+    connect( ui.activeCombo, SIGNAL(editTextChanged(QString)), SLOT(enableButtons(QString)));
     connect( groupModel, SIGNAL( rowsRemoved( const QModelIndex&, int, int ) ), SIGNAL( changed() ) );
     connect( groupModel, SIGNAL( rowsInserted( const QModelIndex&, int, int ) ), SIGNAL( changed() ) );
     connect( groupModel, SIGNAL( rowsRemoved( const QModelIndex&, int, int ) ), SLOT( enableDeleteButton() ) );
@@ -80,6 +81,7 @@ void EnvironmentWidget::enableDeleteButton()
 void EnvironmentWidget::setAsDefault()
 {
     groupModel->changeDefaultGroup( ui.activeCombo->currentText() );
+    enableButtons( ui.activeCombo->currentText() );
     emit changed();
 }
 
@@ -93,10 +95,6 @@ void EnvironmentWidget::loadSettings( KConfig* config )
     QStringList groupList = groupModel->groups();
     kDebug() << "Grouplist:" << groupList << "default group:" << groupModel->defaultGroup();
     ui.activeCombo->addItems( groupList );
-    if( !groupList.contains( groupModel->defaultGroup() ) )
-    {
-        ui.activeCombo->addItem( groupModel->defaultGroup() );
-    }
     int idx = ui.activeCombo->findText( groupModel->defaultGroup() );
     ui.activeCombo->setCurrentIndex( idx );
 }
@@ -166,6 +164,7 @@ void EnvironmentWidget::addGroupClicked()
         return; // same group name cannot be added twice.
     }
     ui.activeCombo->addItem( curText );
+    ui.activeCombo->setCurrentItem( curText );
 }
 
 void EnvironmentWidget::removeGroupClicked()
@@ -179,14 +178,23 @@ void EnvironmentWidget::removeGroupClicked()
     QString curText = ui.activeCombo->currentText();
     groupModel->removeGroup( curText );
     ui.activeCombo->removeItem( idx );
+    ui.activeCombo->setCurrentItem( groupModel->defaultGroup() );
 }
 
 void EnvironmentWidget::activeGroupChanged( int /*idx*/ )
 {
-
     groupModel->setCurrentGroup( ui.activeCombo->currentText() );
+    enableButtons( ui.activeCombo->currentText() );
 }
 
+void EnvironmentWidget::enableButtons( const QString& txt ) 
+{
+    ui.addgrpBtn->setEnabled( !groupModel->groups().contains( txt  ) );
+    ui.removegrpBtn->setEnabled( ( groupModel->groups().contains( txt  ) && groupModel->defaultGroup() != txt ) );
+    ui.setAsDefaultBtn->setEnabled( ( groupModel->groups().contains( txt  ) && groupModel->defaultGroup() != txt ) );
+}
+
+
 }
 
 #include "environmentwidget.moc"
