@@ -155,6 +155,11 @@ public:
     }
 
     QString id() const { return m_factory->id(); }
+    
+    bool viewsWantProgressIndicator() const 
+    {
+            return m_factory->viewsWantProgressIndicator();
+    }
 
 private:
     IToolViewFactory *m_factory;
@@ -227,7 +232,10 @@ void UiController::mainWindowDeleted(MainWindow* mw)
 // should just rename it.
 void UiController::switchToArea(const QString &areaName, SwitchMode switchMode)
 {
-    Q_UNUSED( switchMode );
+    if (switchMode == ThisWindow) {
+        showArea(areaName, activeSublimeWindow());
+        return;
+    }
 
     MainWindow *main = new MainWindow(this);
     // FIXME: what this is supposed to do?
@@ -571,6 +579,15 @@ void UiController::registerStatus(QObject* status)
     mw->registerStatus(status);
 }
 
+void UiController::showErrorMessage(const QString& message, int timeout)
+{
+    Sublime::MainWindow* w = activeSublimeWindow();
+    if (!w) return;
+    MainWindow* mw = qobject_cast<KDevelop::MainWindow*>(w);
+    if (!mw) return;
+    QMetaObject::invokeMethod(mw, "showErrorMessage", Q_ARG(QString, message), Q_ARG(int, timeout));
+}
+
 void UiController::hideAssistant(const KDevelop::IAssistant::Ptr& assistant)
 {
     if(d->currentShownAssistant && d->currentShownAssistant->assistant() == assistant) {
@@ -591,6 +608,11 @@ void UiController::popUpAssistant(const KDevelop::IAssistant::Ptr& assistant)
         return;
     
     Sublime::View* view = d->activeSublimeWindow->activeView();
+    if( !view )
+    {
+        kDebug() << "no active view in mainwindow";
+        return;
+    }
     TextEditorWidget* textWidget = dynamic_cast<TextEditorWidget*>(view->widget());
     if(textWidget && textWidget->editorView()) {
 
