@@ -218,6 +218,7 @@ struct DisconnectMainWindowsFromArea
             foreach(Sublime::MainWindow* window, Core::self()->uiControllerInternal()->mainWindows()) {
                 if(window->area() == area) {
                     mainWindows << window;
+                    oldActiveView = window->activeView();
                     bool hadTempArea = false;
                     foreach(Sublime::Area* tempArea, Core::self()->uiControllerInternal()->areas(window)) {
                         if(tempArea != area) {
@@ -240,6 +241,8 @@ struct DisconnectMainWindowsFromArea
             foreach(Sublime::MainWindow* window, mainWindows) {
                 kDebug() << "changing back";
                 Core::self()->uiControllerInternal()->showArea(m_area, window);
+                if(oldActiveView)
+                    window->activateView(oldActiveView);
                 window->setUpdatesEnabled(true);
             }
         }
@@ -247,6 +250,7 @@ struct DisconnectMainWindowsFromArea
 
     Sublime::Area* m_area;
     QList<Sublime::MainWindow*> mainWindows;
+    QPointer<Sublime::View> oldActiveView;
 };
 
 void loadFileList(QStringList& ret, KConfigGroup group)
@@ -300,7 +304,12 @@ void WorkingSet::loadToArea(Sublime::Area* area, Sublime::AreaIndex* areaIndex,
     
     if(clear) {
         kDebug() << "clearing area with working-set" << area->workingSet();
-        area->clearViews();
+        QSet< QString > files = fileList().toSet();
+        foreach(Sublime::View* view, area->views()) {
+            Sublime::UrlDocument* doc = dynamic_cast<Sublime::UrlDocument*>(view->document());
+            if(!doc || !files.contains(doc->documentSpecifier()))
+                area->closeView(view);
+        }
     }
     
     if(m_id.isEmpty())
@@ -390,7 +399,9 @@ QWidget* WorkingSetController::createSetManagerWidget(MainWindow* parent, bool l
 #endif
 }
 
-WorkingSetWidget::WorkingSetWidget(MainWindow* parent, WorkingSetController* controller, bool mini, Sublime::Area* fixedArea) : QWidget(parent), m_mini(mini), m_mainWindow(parent), m_fixedArea(fixedArea) {
+WorkingSetWidget::WorkingSetWidget(MainWindow* parent, WorkingSetController* controller, bool mini, Sublime::Area* fixedArea)
+	: QWidget(parent), m_fixedArea(fixedArea), m_mini(mini), m_mainWindow(parent)
+{
     m_layout = new QHBoxLayout(this);
     m_layout->setMargin(0);
     if(!m_fixedArea)
@@ -567,11 +578,7 @@ void WorkingSetToolButton::buttonTriggered()
         return;
     
     if(mainWindow()->area()->workingSet() == m_set->id()) {
-        //Create a new working-set
-//         if(!m_mini) {
-                mainWindow()->area()->setWorkingSet(QString());
-//         }else{
-//         }
+        closeSet();
     }else{
         mainWindow()->area()->setWorkingSet(m_set->id());
     }
@@ -727,6 +734,8 @@ WorkingSetToolTipWidget::WorkingSetToolTipWidget(QWidget* parent, WorkingSet* se
     connect(set, SIGNAL(setChangedSignificantly()), SLOT(updateFileButtons()));
     connect(Core::self()->workingSetControllerInternal()->getWorkingSet(mainwindow->area()->workingSet()), SIGNAL(setChangedSignificantly()), SLOT(updateFileButtons()));
     connect(mainwindow->area(), SIGNAL(changedWorkingSet(Sublime::Area*,QString,QString)), SLOT(updateFileButtons()), Qt::QueuedConnection);
+    
+    QMetaObject::invokeMethod(this, "updateFileButtons");
 }
 
 
@@ -752,7 +761,7 @@ void WorkingSetToolTipWidget::updateFileButtons()
         }
     }
     
-    m_mergeButton->setEnabled(!allOpen && mainWindow->area()->workingSet() != m_set->id());
+    m_mergeButton->setEnabled(!allOpen);
     m_subtractButton->setEnabled(!noneOpen && mainWindow->area()->workingSet() != m_set->id());
     m_deleteButton->setEnabled(!m_set->hasConnectedAreas());
     
