@@ -42,13 +42,14 @@
 using namespace Sublime;
 
 IdealToolButton::IdealToolButton(Qt::DockWidgetArea area, QWidget *parent)
-    : QToolButton(parent), _area(area)
+    : KAnimatedButton(parent), _area(area), showingIndicator( false )
 {
     setFocusPolicy(Qt::NoFocus);
     KAcceleratorManager::setNoAccel(this);
     setCheckable(true);
     setAutoRaise(true);
     setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
+    setIcons( "process-working-kde" );
 }
 
 Qt::Orientation IdealToolButton::orientation() const
@@ -59,6 +60,29 @@ Qt::Orientation IdealToolButton::orientation() const
     return Qt::Horizontal;
 }
 
+void IdealToolButton::hideProgressIndicator()
+{
+    stop();
+    setIcon( normalIcon );
+    showingIndicator = false;
+}
+
+void IdealToolButton::showProgressIndicator()
+{
+    setIcons( "process-working-kde" );
+    showingIndicator = true;
+    start();
+}
+
+void IdealToolButton::updateNormalIcon( const QIcon& icon )
+{
+    normalIcon = icon;
+    if( !showingIndicator )
+    {
+        setIcon( icon );
+    }
+}
+
 QSize IdealToolButton::sizeHint() const
 {
     ensurePolished();
@@ -70,12 +94,27 @@ QSize IdealToolButton::sizeHint() const
 
     const int charWidth = fm.width(QLatin1Char('x'));
 
-    QSize textSize = fm.size(Qt::TextShowMnemonic, opt.text);
-    textSize.rwidth() += 2 * charWidth;
+    QSize textSize;
+    // No text size if we're having icon-only button
+    if (toolButtonStyle() != Qt::ToolButtonIconOnly) {
+        textSize = fm.size(Qt::TextShowMnemonic, opt.text);
+        textSize.rwidth() += 2 * charWidth;
+    }
 
     const int spacing = 2; // ### FIXME
-    int width = 4 + textSize.width() + opt.iconSize.width() + spacing;
-    int height = 4 + qMax(textSize.height(), opt.iconSize.height());
+    int iconwidth = 0, iconheight = 0;
+    // No icon size if we're drawing text only
+    if (toolButtonStyle() != Qt::ToolButtonTextOnly) {
+        if (_area == Qt::TopDockWidgetArea || _area == Qt::BottomDockWidgetArea) {
+            iconwidth = opt.iconSize.width();
+            iconheight = opt.iconSize.height();
+        } else {
+            iconwidth = opt.iconSize.height();
+            iconheight = opt.iconSize.width();
+        }
+    }
+    int width = 4 + textSize.width() + iconwidth + spacing;
+    int height = 4 + qMax(textSize.height(), iconheight) + spacing;
 
     if (orientation() == Qt::Vertical)
         return QSize(height, width);
@@ -90,7 +129,16 @@ void IdealToolButton::paintEvent(QPaintEvent *event)
     } else {
         QStyleOptionToolButton opt;
         initStyleOption(&opt);
-	opt.rect.setSize(QSize(opt.rect.height(), opt.rect.width()));
+        // If we're drawing icons only it looks better if they're not rotated
+        QPixmap ic = icon().pixmap(QSize(16,16), QIcon::Normal, QIcon::On);
+        QTransform tf;
+        if(_area == Qt::LeftDockWidgetArea) {
+            tf = tf.rotate(90);
+        } else {
+            tf = tf.rotate(-90);
+        }
+        opt.icon = ic.transformed( tf, Qt::FastTransformation );
+        opt.rect.setSize(QSize(opt.rect.height(), opt.rect.width()));
 
         QPixmap pix(opt.rect.width(), opt.rect.height());
 	QStylePainter painter(&pix, this);
@@ -145,7 +193,6 @@ IdealButtonBarWidget::IdealButtonBarWidget(Qt::DockWidgetArea area, IdealMainWid
 KAction *IdealButtonBarWidget::addWidget(const QString& title, IdealDockWidget *dock,
                                          Area *area, View *view)
 {
-    kDebug() << "adding widget";
     KAction *action = new KAction(this);
     action->setCheckable(true);
     action->setText(title);
@@ -177,6 +224,8 @@ QWidget* IdealButtonBarWidget::corner()
 
 void IdealButtonBarWidget::removeAction(QAction * action)
 {
+    disconnect(_widgets[action]->view(), SIGNAL(showProgressIndicator()), _buttons[action], 0);
+    disconnect(_widgets[action]->view(), SIGNAL(hideProgressIndicator()), _buttons[action], 0);
     _widgets.remove(action);
     delete _buttons.take(action);
     delete action;
@@ -226,12 +275,14 @@ void IdealButtonBarWidget::actionEvent(QActionEvent *event)
             _buttons.insert(action, button);
 
             button->setText(action->text());
-            button->setIcon(action->icon());
+            button->updateNormalIcon(action->icon());
             button->setShortcut(QKeySequence());
             button->setChecked(action->isChecked());
             layout()->addWidget(button);
             connect(action, SIGNAL(toggled(bool)), SLOT(actionToggled(bool)));
             connect(button, SIGNAL(toggled(bool)), action, SLOT(setChecked(bool)));
+            connect(_widgets[action]->view(), SIGNAL(showProgressIndicator()), button, SLOT(showProgressIndicator()));
+            connect(_widgets[action]->view(), SIGNAL(hideProgressIndicator()), button, SLOT(hideProgressIndicator()));
         }
     } break;
 
@@ -253,7 +304,7 @@ void IdealButtonBarWidget::actionEvent(QActionEvent *event)
     case QEvent::ActionChanged: {
 	if (IdealToolButton *button = _buttons.value(action)) {
 	    button->setText(action->text());
-	    button->setIcon(action->icon());
+	    button->updateNormalIcon(action->icon());
 	    button->setShortcut(QKeySequence());
 	    Q_ASSERT(_widgets.contains(action));
 	    _widgets[action]->setWindowTitle(action->text());
@@ -444,6 +495,7 @@ IdealMainWidget::IdealMainWidget(MainWindow* parent, KActionCollection* ac)
     : QWidget(parent)
     , m_centralWidgetFocusing(false), m_switchingDocksShown(false)
 {
+    Q_UNUSED(ac);
     leftBarWidget = new IdealButtonBarWidget(Qt::LeftDockWidgetArea, this);
     leftBarWidget->hide();
 
@@ -583,7 +635,6 @@ bool IdealMainWidget::someDockMaximized()
 // helper for toggleDocksShown
 void IdealMainWidget::restorePreviouslyShownDocks()
 {
-    kDebug() << "";
     foreach(View* v, m_previouslyShownDocks) {
         if (v && m_view_to_action.contains(v)) {
             m_view_to_action[v]->setChecked(true);
@@ -594,7 +645,6 @@ void IdealMainWidget::restorePreviouslyShownDocks()
 // helper for toggleDocksShown
 void IdealMainWidget::hideAllShownDocks()
 {
-    kDebug() << "";
     m_previouslyShownDocks.clear();
     QMapIterator<View*, QAction*> it(m_view_to_action);
     while(it.hasNext()) {
@@ -609,7 +659,6 @@ void IdealMainWidget::hideAllShownDocks()
 
 void IdealMainWidget::toggleDocksShown()
 {
-    kDebug() << "";
     if (m_switchingDocksShown || someDockMaximized()) {
         return;
     }
