@@ -69,9 +69,16 @@ void MainWindow::setupAreaSelector() {
     if(areas.isEmpty())
         areas = controller()->defaultAreas();
     
+    QSet<QString> hadAreaName;
+    
     for(int a = 0; a < areas.size(); ++a) {
         Area* theArea = areas[a];
         
+        if(hadAreaName.contains(theArea->objectName()))
+            continue;
+        
+        hadAreaName.insert(theArea->objectName());
+        
         if(theArea->objectName() == area()->objectName()) {
             currentIndex = a;
         }
@@ -109,7 +116,12 @@ void MainWindow::setArea(Area *area)
 
     d->area = area;
     d->reconstruct();
-    d->activateFirstVisibleView();
+    
+    if(d->area->activeView())
+        activateView(d->area->activeView());
+    else
+        d->activateFirstVisibleView();
+    
     initializeStatusBar();
     emit areaChanged(area);
     d->ignoreDockShown = false;
@@ -181,6 +193,7 @@ void MainWindow::activateView(View *view)
     d->viewContainers[view]->setCurrentWidget(view->widget());
 
     setActiveView(view);
+    d->area->setActiveView(view);
 }
 
 void MainWindow::setActiveView(View *view)
@@ -269,6 +282,9 @@ void MainWindow::loadSettings()
         state = QByteArray::fromBase64(state);
         // One day will need to load the version number, but for now, assume 0
         restoreState(state);
+    } else {
+        // If there's no state we use a default size of 700x600
+        resize(700,600);
     }
     
     KConfigGroup uiGroup = KGlobal::config()->group("UiSettings");
