@@ -22,12 +22,21 @@
 #include <QMap>
 #include <QObject>
 #include <QSet>
+#include <QTabBar>
+#include <QHBoxLayout>
+#include <QLabel>
 
 #include "area.h"
 #include "sublimedefs.h"
 
 #include "mainwindow.h"
 #include "ideallayout.h"
+#include <qmenubar.h>
+#include <qstylepainter.h>
+#include <qstyleoption.h>
+#include <qevent.h>
+#include <KColorScheme>
+#include "blur.h"
 
 class QMenu;
 class QAction;
@@ -43,10 +52,89 @@ class Controller;
 class AreaIndex;
 class IdealMainWidget;
 
+#if QT_VERSION >= 0x040500
+class AreaTabButton : public QWidget {
+    public:
+    AreaTabButton(QString text, QIcon icon, uint iconSize, QWidget* parent, bool isCurrent, QWidget* _customButtonWidget) ;
+    
+    QLabel* iconLabel;
+    QLabel* textLabel;
+    QWidget* customButtonWidget;
+
+    void setIsCurrent ( bool arg1 );
+
+    private:
+    bool m_isCurrent;
+};
+
+class AreaTabBar : public QTabBar {
+    public:
+    AreaTabBar(QWidget* parent) ;
+
+    //Non-vertual overload: Respected by mainwindow.cpp
+    void setCurrentIndex(int index) ;
+    
+    virtual QSize tabSizeHint ( int index ) const ;
+    
+    void clearTabs() {
+        while(count())
+            removeTab(0);
+        buttons.clear();
+        areaIds.clear();
+    }
+    
+    QString areaId(int index) const {
+        return areaIds[index];
+    }
+    
+    void addCustomTab(QString text, QIcon icon, bool isCurrent, QString areaId, QWidget* customButtonWidget) {
+        areaIds << areaId;
+        buttons << new AreaTabButton(text, icon, 16, this, isCurrent, customButtonWidget);
+        addTab(QString());
+        setTabButton(count()-1, LeftSide, buttons.last());
+    }
+    
+    virtual void paintEvent ( QPaintEvent* );
+    
+    private:
+    QList<QString> areaIds;
+    QList<AreaTabButton*> buttons;
+    int m_currentIndex;
+};
+
+class AreaTabWidget : public QWidget {
+    public:
+    AreaTabWidget(QMenuBar* parent) ;
+    
+    virtual QSize sizeHint() const ;
+    
+    virtual void paintEvent(QPaintEvent *ev);
+    
+    QMenuBar* bar() const {
+        return static_cast<QMenuBar*>(parent());
+    }
+    
+    ///The widget is owned by this tab-widget
+    void setTabSideWidget(QWidget* widget) {
+        if(areaSideWidget)
+            delete areaSideWidget;
+        areaSideWidget = widget;
+        m_leftLayout->insertWidget(0, areaSideWidget);
+    }
+
+    AreaTabBar* tabBar;
+    QWidget* areaSideWidget;
+
+    QHBoxLayout* m_layout;
+    QVBoxLayout* m_leftLayout;
+};
+#endif
+
 class MainWindowPrivate: public QObject {
     Q_OBJECT
 public:
     MainWindowPrivate(MainWindow *w, Controller* controller);
+    ~MainWindowPrivate();
 
     /**Use this to create tool views for an area.*/
     class IdealToolViewCreator {
@@ -73,8 +161,6 @@ public:
 
     void activateFirstVisibleView();
 
-    void setStatusIcon(View* view, const QIcon& icon);
-
     Controller *controller;
     Area *area;
     QList<View*> docks;
@@ -85,6 +171,9 @@ public:
     View *activeToolView;
 
     QWidget *centralWidget;
+#if QT_VERSION >= 0x040500
+    AreaTabWidget* areaSwitcher;
+#endif
 
     IdealMainWidget *idealMainWidget;
     int ignoreDockShown;
@@ -97,6 +186,9 @@ public slots:
     void toolViewAdded(Sublime::View *toolView, Sublime::Position position);
     void aboutToRemoveToolView(Sublime::View *toolView, Sublime::Position position);
     void toolViewMoved(Sublime::View *toolView, Sublime::Position position);
+    void toggleArea(int index);
+
+    void setTabBarLeftCornerWidget(QWidget* widget);
 
 private slots:
     void switchToArea(QAction *action);
@@ -130,6 +222,7 @@ private:
 
     QMap<Area*, QAction*> m_areaActions;
     QMap<QAction*, Area*> m_actionAreas;
+    QPointer<QWidget> m_leftTabbarCornerWidget;
 };
 
 }
