@@ -40,6 +40,7 @@ public:
     virtual QWidget* create(ToolDocument *doc, QWidget *parent = 0) = 0;
     virtual QList<QAction*> toolBarActions( QWidget* viewWidget ) const = 0;
     virtual QString id() const = 0;
+    virtual bool viewsWantProgressIndicator() const = 0;
 };
 
 /**
@@ -55,6 +56,7 @@ public:
     }
     virtual QList<QAction*> toolBarActions( QWidget* ) const { return QList<QAction*>(); }
     virtual QString id() const { return m_id; }
+    virtual bool viewsWantProgressIndicator() const { return false; }
 private:
     QString m_id;
 };
@@ -76,11 +78,16 @@ public:
 protected:
     virtual QWidget *createViewWidget(QWidget *parent = 0);
     ToolFactory *factory() const;
+    
+    /**Creates and returns the new view. Reimplement in subclasses to instantiate
+    views of derived from Sublime::View classes.*/
+    virtual View *newView(Document *doc);
 
 private:
     struct ToolDocumentPrivate * const d;
     
     friend class View;
+    friend class ToolView;
 };
 
 }
