@@ -21,14 +21,14 @@
 #ifndef VERITAS_TESTRESULT_H
 #define VERITAS_TESTRESULT_H
 
-#include "veritas/veritasexport.h"
+#include "veritasexport.h"
 
 #include <QtCore/QList>
 #include <QtCore/QString>
 #include <QtCore/QVariant>
 #include <QtCore/QFileInfo>
 #include <QtCore/QByteArray>
-#include <KUrl>
+#include <KDE/KUrl>
 
 namespace Veritas
 {
